#!/usr/bin/python
#
# Copyright (c) 2019 Zim Kalinowski, (@zikalino)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_hdinsightcluster
version_added: "0.1.2"
short_description: Manage Azure HDInsight Cluster instance
description:
    - Create, update and delete instance of Azure HDInsight Cluster.

options:
    resource_group:
        description:
            - The name of the resource group.
        required: True
        type: str
    name:
        description:
            - The name of the cluster.
        required: True
        type: str
    location:
        description:
            - Resource location. If not set, location from the resource group will be used as default.
        type: str
    cluster_version:
        description:
            - The version of the cluster. For example C(3.6).
        type: str
    os_type:
        description:
            - The type of operating system.
        type: str
        choices:
            - 'linux'
    tier:
        description:
            - The cluster tier.
        type: str
        choices:
            - 'standard'
            - 'premium'
    cluster_definition:
        description:
            - The cluster definition.
        type: dict
        suboptions:
            kind:
                description:
                    - The type of cluster.
                type: str
                choices:
                    - hadoop
                    - spark
                    - hbase
                    - storm
            gateway_rest_username:
                description:
                    - Gateway REST user name.
                type: str
            gateway_rest_password:
                description:
                    - Gateway REST password.
                type: str
    compute_profile_roles:
        description:
            - The list of roles in the cluster.
        type: list
        elements: dict
        suboptions:
            name:
                description:
                    - The name of the role.
                type: str
                choices:
                    - 'headnode'
                    - 'workernode'
                    - 'zookeepernode'
            min_instance_count:
                description:
                    - The minimum instance count of the cluster.
                type: int
            target_instance_count:
                description:
                    - The instance count of the cluster.
                type: int
            vm_size:
                description:
                    - The size of the VM.
                type: str
            linux_profile:
                description:
                    - The Linux OS profile.
                type: dict
                suboptions:
                    username:
                        description:
                            - SSH user name.
                        type: str
                    password:
                        description:
                            - SSH password.
                        type: str
    identity:
        description:
            - Identity for the HDInsight Cluster.
        type: dict
        version_added: '3.0.0'
        suboptions:
            type:
                description:
                    - Type of the managed identity
                choices:
                    - SystemAssigned
                    - UserAssigned
                    - SystemAssigned, UserAssigned
                    - None
                default: None
                type: str
            user_assigned_identities:
                description:
                    - User Assigned Managed Identities and its options
                required: false
                type: dict
                default: {}
                suboptions:
                    id:
                        description:
                            - List of the user assigned identities IDs associated to the HDInsight Cluster
                        required: false
                        type: list
                        elements: str
                        default: []
                    append:
                        description:
                            - If the list of identities has to be appended to current identities (true) or if it has to replace current identities (false)
                        required: false
                        type: bool
                        default: True
    storage_accounts:
        description:
            - The list of storage accounts in the cluster.
        type: list
        elements: dict
        suboptions:
            name:
                description:
                    - Blob storage endpoint. For example storage_account_name.blob.core.windows.net.
                type: str
            is_default:
                description:
                    - Whether or not the storage account is the default storage account.
                type: bool
            container:
                description:
                    - The container in the storage account.
                type: str
            key:
                description:
                    - The storage account access key.
                type: str
    state:
        description:
            - Assert the state of the cluster.
            - Use C(present) to create or update a cluster and C(absent) to delete it.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Zim Kalinowski (@zikalino)

'''

EXAMPLES = '''
- name: Create instance of HDInsight Cluster
  azure_rm_hdinsightcluster:
    resource_group: myResourceGroup
    name: myCluster
    location: eastus2
    cluster_version: 3.6
    os_type: linux
    tier: standard
    cluster_definition:
      kind: spark
      gateway_rest_username: http-user
      gateway_rest_password: "{{ password }}"
    storage_accounts:
      - name: myStorageAccount.blob.core.windows.net
        is_default: true
        container: myContainer
        key: GExmaxH4lDNdHA9nwAsCt8t4AOQas2y9vXQP1kKALTram7Q3/5xLVIab3+nYG1x63Xyak9/VXxQyNBHA9pDWw==
    compute_profile_roles:
      - name: headnode
        target_instance_count: 2
        vm_size: Standard_D3
        linux_profile:
          username: sshuser
          password: "{{ password }}"
      - name: workernode
        target_instance_count: 2
        vm_size: Standard_D3
        linux_profile:
          username: sshuser
          password: "{{ password }}"
'''

RETURN = '''
id:
    description:
        - Fully qualified resource id of the cluster.
    returned: always
    type: str
    sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.HDInsight/clusters/myCluster
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt

try:
    from azure.core.polling import LROPoller
    from azure.core.exceptions import ResourceNotFoundError
    from azure.mgmt.hdinsight import HDInsightManagementClient
    from azure.mgmt.hdinsight.models import ClusterIdentity, UserAssignedIdentity
except ImportError:
    # This is handled in azure_rm_common
    pass


class Actions:
    NoAction, Create, Update, Delete = range(4)


class AzureRMClusters(AzureRMModuleBaseExt):
    """Configuration class for an Azure RM Cluster resource"""

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True
            ),
            location=dict(
                type='str'
            ),
            cluster_version=dict(
                type='str'
            ),
            identity=dict(
                type='dict',
                options=self.managed_identity_multiple_spec
            ),
            os_type=dict(
                type='str',
                choices=['linux']
            ),
            tier=dict(
                type='str',
                choices=['standard',
                         'premium']
            ),
            cluster_definition=dict(
                type='dict'
            ),
            compute_profile_roles=dict(
                type='list',
                elements='dict',
                options=dict(
                    name=dict(type='str', choices=['headnode', 'workernode', 'zookeepernode']),
                    min_instance_count=dict(type='int'),
                    target_instance_count=dict(type='int'),
                    vm_size=dict(type='str'),
                    linux_profile=dict(
                        type='dict',
                        options=dict(
                            username=dict(type='str'),
                            password=dict(type='str', no_log=True)
                        )
                    ),
                )
            ),
            storage_accounts=dict(
                type='list',
                elements='dict',
                options=dict(
                    name=dict(type='str'),
                    is_default=dict(type='bool'),
                    container=dict(type='str'),
                    key=dict(type='str', no_log=True)
                )
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )

        self.resource_group = None
        self.name = None
        self.parameters = dict()

        self.results = dict(changed=False)
        self.mgmt_client = None
        self.state = None
        self.to_do = Actions.NoAction
        self.tags_changed = False
        self.new_instance_count = None
        self.identity = None
        self._managed_identity = None

        super(AzureRMClusters, self).__init__(derived_arg_spec=self.module_arg_spec,
                                              supports_check_mode=True,
                                              supports_tags=True)

    @property
    def managed_identity(self):
        if not self._managed_identity:
            self._managed_identity = {"identity": ClusterIdentity,
                                      "user_assigned": UserAssignedIdentity
                                      }
        return self._managed_identity

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs[key] is not None:
                self.parameters[key] = kwargs[key]

        dict_expand(self.parameters, ['cluster_version'], 'properties')
        dict_camelize(self.parameters, ['os_type'], True)
        dict_expand(self.parameters, ['os_type'], 'properties')
        dict_camelize(self.parameters, ['tier'], True)
        dict_expand(self.parameters, ['tier'], 'properties')

        dict_rename(self.parameters, ['cluster_definition', 'gateway_rest_username'], 'restAuthCredential.username')
        dict_rename(self.parameters, ['cluster_definition', 'gateway_rest_password'], 'restAuthCredential.password')
        dict_expand(self.parameters, ['cluster_definition', 'restAuthCredential.username'], 'gateway')
        dict_expand(self.parameters, ['cluster_definition', 'restAuthCredential.password'], 'gateway')
        dict_expand(self.parameters, ['cluster_definition', 'gateway'], 'configurations')

        dict_expand(self.parameters, ['cluster_definition'], 'properties')
        dict_expand(self.parameters, ['compute_profile_roles', 'vm_size'], 'hardware_profile')
        dict_rename(self.parameters, ['compute_profile_roles', 'linux_profile'], 'linux_operating_system_profile')
        dict_expand(self.parameters, ['compute_profile_roles', 'linux_operating_system_profile'], 'os_profile')
        dict_rename(self.parameters, ['compute_profile_roles'], 'roles')
        dict_expand(self.parameters, ['roles'], 'compute_profile')
        dict_expand(self.parameters, ['compute_profile'], 'properties')
        dict_rename(self.parameters, ['storage_accounts'], 'storageaccounts')
        dict_expand(self.parameters, ['storageaccounts'], 'storage_profile')
        dict_expand(self.parameters, ['storage_profile'], 'properties')

        response = None

        self.mgmt_client = self.get_mgmt_svc_client(HDInsightManagementClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager)

        resource_group = self.get_resource_group(self.resource_group)

        if "location" not in self.parameters:
            self.parameters["location"] = resource_group.location

        old_response = self.get_cluster()

        curr_identity = old_response.get('identity') if old_response else None

        self.parameters['identity'] = None
        if self.identity:
            self.update_identity, identity_result = self.update_managed_identity(curr_identity=curr_identity,
                                                                                 new_identity=self.identity)
            self.parameters['identity'] = identity_result.as_dict()

        if not old_response:
            self.log("Cluster instance doesn't exist")
            if self.state == 'absent':
                self.log("Old instance didn't exist")
            else:
                self.to_do = Actions.Create
        else:
            self.log("Cluster instance already exists")
            if self.state == 'absent':
                self.to_do = Actions.Delete
            elif self.state == 'present':
                compare_result = {}
                if (not default_compare(self.parameters, old_response, '', compare_result)):
                    if compare_result.pop('/properties/compute_profile/roles/*/target_instance_count', False):
                        # check if it's workernode
                        new_count = 0
                        old_count = 0
                        for role in self.parameters['properties']['compute_profile']['roles']:
                            if role['name'] == 'workernode':
                                new_count = role['target_instance_count']
                        for role in old_response['properties']['compute_profile']['roles']:
                            if role['name'] == 'workernode':
                                old_count = role['target_instance_count']
                        if old_count != new_count:
                            self.new_instance_count = new_count
                            self.to_do = Actions.Update
                    if compare_result.pop('/tags', False):
                        self.to_do = Actions.Update
                        self.tags_changed = True
                    if compare_result:
                        for k in compare_result.keys():
                            self.module.warn("property '" + k + "' cannot be updated (" + compare_result[k] + ")")
                        self.module.warn("only tags and target_instance_count can be updated")

        if (self.to_do == Actions.Create) or (self.to_do == Actions.Update):
            self.log("Need to Create / Update the Cluster instance")
            self.results['changed'] = True
            if self.check_mode:
                return self.results
            response = self.create_update_cluster()
            self.log("Creation / Update done")
        elif self.to_do == Actions.Delete:
            self.log("Cluster instance deleted")
            self.results['changed'] = True
            if self.check_mode:
                return self.results
            self.delete_cluster()
        else:
            self.log("Cluster instance unchanged")
            self.results['changed'] = False
            response = old_response

        if self.state == 'present':
            self.results.update(self.format_item(response))
        return self.results

    def create_update_cluster(self):
        '''
        Creates or updates Cluster with the specified configuration.

        :return: deserialized Cluster instance state dictionary
        '''
        self.log("Creating / Updating the Cluster instance {0}".format(self.name))

        try:
            if self.to_do == Actions.Create:
                response = self.mgmt_client.clusters.begin_create(resource_group_name=self.resource_group,
                                                                  cluster_name=self.name,
                                                                  parameters=self.parameters)
                if isinstance(response, LROPoller):
                    response = self.get_poller_result(response)
            else:
                if self.tags_changed:
                    response = self.mgmt_client.clusters.update(resource_group_name=self.resource_group,
                                                                cluster_name=self.name,
                                                                parameters={'tags': self.parameters.get('tags')})
                    if isinstance(response, LROPoller):
                        response = self.get_poller_result(response)
                if self.new_instance_count:
                    response = self.mgmt_client.clusters.begin_resize(resource_group_name=self.resource_group,
                                                                      cluster_name=self.name,
                                                                      role_name='workernode',
                                                                      parameters={'target_instance_count': self.new_instance_count})
                    if isinstance(response, LROPoller):
                        response = self.get_poller_result(response)
        except Exception as exc:
            self.fail("Error creating or updating Cluster instance: {0}".format(str(exc)))
        return response.as_dict() if response else {}

    def delete_cluster(self):
        '''
        Deletes specified Cluster instance in the specified subscription and resource group.

        :return: True
        '''
        self.log("Deleting the Cluster instance {0}".format(self.name))
        try:
            response = self.mgmt_client.clusters.begin_delete(resource_group_name=self.resource_group,
                                                              cluster_name=self.name)
        except Exception as e:
            self.fail("Error deleting the Cluster instance: {0}".format(str(e)))

        return True

    def get_cluster(self):
        '''
        Gets the properties of the specified Cluster.

        :return: deserialized Cluster instance state dictionary
        '''
        self.log("Checking if the Cluster instance {0} is present".format(self.name))
        found = False
        try:
            response = self.mgmt_client.clusters.get(resource_group_name=self.resource_group,
                                                     cluster_name=self.name)
            found = True
            self.log("Response : {0}".format(response))
            self.log("Cluster instance : {0} found".format(response.name))
        except ResourceNotFoundError as e:
            self.log('Did not find the Cluster instance.')
        if found is True:
            return response.as_dict()

        return False

    def format_item(self, d):
        d = {
            'id': d.get('id', None)
        }
        return d


def default_compare(new, old, path, result):
    if new is None:
        match = True
    elif isinstance(new, dict):
        match = True
        if not isinstance(old, dict):
            result[path] = 'old dict is null'
            match = False
        else:
            for k in new.keys():
                if not default_compare(new.get(k), old.get(k, None), path + '/' + k, result):
                    match = False
    elif isinstance(new, list):
        if not isinstance(old, list) or len(new) != len(old):
            result[path] = 'length is different or null'
            match = False
        elif len(old) == 0:
            match = True
        else:
            match = True
            if isinstance(old[0], dict):
                key = None
                if 'id' in old[0] and 'id' in new[0]:
                    key = 'id'
                elif 'name' in old[0] and 'name' in new[0]:
                    key = 'name'
                else:
                    key = list(old[0])[0]
                new = sorted(new, key=lambda x: x.get(key, ''))
                old = sorted(old, key=lambda x: x.get(key, ''))
            else:
                new = sorted(new)
                old = sorted(old)
            for i in range(len(new)):
                if not default_compare(new[i], old[i], path + '/*', result):
                    match = False
        return match
    else:
        if path.endswith('password'):
            match = True
        else:
            if path == '/location' or path.endswith('location_name'):
                new = new.replace(' ', '').lower()
                old = new.replace(' ', '').lower()
            if new == old:
                match = True
            else:
                result[path] = str(new) + ' != ' + str(old)
                match = False
    return match


def dict_camelize(d, path, camelize_first):
    if isinstance(d, list):
        for i in range(len(d)):
            dict_camelize(d[i], path, camelize_first)
    elif isinstance(d, dict):
        if len(path) == 1:
            old_value = d.get(path[0], None)
            if old_value is not None:
                d[path[0]] = _snake_to_camel(old_value, camelize_first)
        else:
            sd = d.get(path[0], None)
            if sd is not None:
                dict_camelize(sd, path[1:], camelize_first)


def dict_upper(d, path):
    if isinstance(d, list):
        for i in range(len(d)):
            dict_upper(d[i], path)
    elif isinstance(d, dict):
        if len(path) == 1:
            old_value = d.get(path[0], None)
            if old_value is not None:
                d[path[0]] = old_value.upper()
        else:
            sd = d.get(path[0], None)
            if sd is not None:
                dict_upper(sd, path[1:])


def dict_rename(d, path, new_name):
    if isinstance(d, list):
        for i in range(len(d)):
            dict_rename(d[i], path, new_name)
    elif isinstance(d, dict):
        if len(path) == 1:
            old_value = d.pop(path[0], None)
            if old_value is not None:
                d[new_name] = old_value
        else:
            sd = d.get(path[0], None)
            if sd is not None:
                dict_rename(sd, path[1:], new_name)


def dict_expand(d, path, outer_dict_name):
    if isinstance(d, list):
        for i in range(len(d)):
            dict_expand(d[i], path, outer_dict_name)
    elif isinstance(d, dict):
        if len(path) == 1:
            old_value = d.pop(path[0], None)
            if old_value is not None:
                d[outer_dict_name] = d.get(outer_dict_name, {})
                d[outer_dict_name][path[0]] = old_value
        else:
            sd = d.get(path[0], None)
            if sd is not None:
                dict_expand(sd, path[1:], outer_dict_name)


def _snake_to_camel(snake, capitalize_first=False):
    if capitalize_first:
        return ''.join(x.capitalize() or '_' for x in snake.split('_'))
    else:
        return snake.split('_')[0] + ''.join(x.capitalize() or '_' for x in snake.split('_')[1:])


def main():
    """Main execution"""
    AzureRMClusters()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2023, Akini Ross <akinross@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_access_span_filter_group_entry
short_description: Manage Access SPAN filter group entries (span:FilterEntry)
description:
- Manage Access SPAN filter group entries on Cisco ACI fabrics.
options:
  filter_group:
    description:
    - The name of the Access SPAN filter group.
    type: str
  source_ip:
    description:
    - The source IP Prefix.
    type: str
  destination_ip:
    description:
    - The destination IP Prefix.
    type: str
  first_src_port:
    description:
    - The first source port (from port).
    - Accepted values are any valid TCP/UDP port range.
    - The APIC defaults to C(unspecified) when unset during creation.
    type: str
  last_src_port:
    description:
    - The last source port (to port).
    - Accepted values are any valid TCP/UDP port range.
    - The APIC defaults to C(unspecified) when unset during creation.
    type: str
  first_dest_port:
    description:
    - The first destination port (from port).
    - Accepted values are any valid TCP/UDP port range.
    - The APIC defaults to C(unspecified) when unset during creation.
    type: str
  last_dest_port:
    description:
    - The last destination port (to port).
    - Accepted values are any valid TCP/UDP port range.
    - The APIC defaults to C(unspecified) when unset during creation.
    type: str
  ip_protocol:
    description:
    - The IP Protocol.
    - The APIC defaults to C(unspecified) when unset during creation.
    type: str
    choices: [ eigrp, egp, icmp, icmpv6, igmp, igp, l2tp, ospfigp, pim, tcp, udp, unspecified ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation

notes:
- The C(filter_group) used must exist before using this module in your playbook.
  The M(cisco.aci.aci_access_span_filter_group) module can be used for this.
seealso:
- module: cisco.aci.aci_access_span_filter_group
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(span:FilterEntry).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Akini Ross (@akinross)
"""

EXAMPLES = r"""
- name: Add a Access SPAN filter entry
  cisco.aci.aci_access_span_filter_group_entry:
    host: apic
    username: admin
    password: SomeSecretPassword
    filter_group: group1
    source_ip: 1.1.1.1
    destination_ip: 2.2.2.2
    state: present
  delegate_to: localhost

- name: Remove a Access SPAN filter entry
  cisco.aci.aci_access_span_filter_group_entry:
    host: apic
    username: admin
    password: SomeSecretPassword
    filter_group: group1
    source_ip: 1.1.1.1
    destination_ip: 2.2.2.2
    state: absent
  delegate_to: localhost

- name: Query a Access SPAN filter group
  cisco.aci.aci_access_span_filter_group_entry:
    host: apic
    username: admin
    password: SomeSecretPassword
    filter_group: group1
    source_ip: 1.1.1.1
    destination_ip: 2.2.2.2
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all Access SPAN filter groups
  cisco.aci.aci_access_span_filter_group_entry:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec
from ansible_collections.cisco.aci.plugins.module_utils.constants import VALID_IP_PROTOCOLS, FILTER_PORT_MAPPING


def get_port_value(port):
    return FILTER_PORT_MAPPING.get(port, port) if port else "unspecified"


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        filter_group=dict(type="str"),  # Not required for querying all objects
        source_ip=dict(type="str"),  # Not required for querying all objects
        destination_ip=dict(type="str"),  # Not required for querying all objects
        first_src_port=dict(type="str"),
        last_src_port=dict(type="str"),
        first_dest_port=dict(type="str"),
        last_dest_port=dict(type="str"),
        ip_protocol=dict(type="str", choices=VALID_IP_PROTOCOLS),
        name_alias=dict(type="str"),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["filter_group", "source_ip", "destination_ip"]],
            ["state", "present", ["filter_group", "source_ip", "destination_ip"]],
        ],
    )

    aci = ACIModule(module)

    filter_group = module.params.get("filter_group")
    source_ip = module.params.get("source_ip")
    destination_ip = module.params.get("destination_ip")
    first_src_port = module.params.get("first_src_port")
    last_src_port = module.params.get("last_src_port")
    first_dest_port = module.params.get("first_dest_port")
    last_dest_port = module.params.get("last_dest_port")
    ip_protocol = module.params.get("ip_protocol")
    state = module.params.get("state")
    name_alias = module.params.get("name_alias")

    aci.construct_url(
        root_class=dict(
            aci_class="infra",
            aci_rn="infra",
        ),
        subclass_1=dict(
            aci_class="spanFilterGrp",
            aci_rn="filtergrp-{0}".format(filter_group),
            module_object=filter_group,
            target_filter={"name": filter_group},
        ),
        subclass_2=dict(
            aci_class="spanFilterEntry",
            aci_rn="proto-{0}-src-[{1}]-dst-[{2}]-srcPortFrom-{3}-srcPortTo-{4}-dstPortFrom-{5}-dstPortTo-{6}".format(
                ip_protocol if module.params.get("ip_protocol") else "unspecified",
                source_ip,
                destination_ip,
                get_port_value(first_src_port),
                get_port_value(last_src_port),
                get_port_value(first_dest_port),
                get_port_value(last_dest_port),
            ),
            target_filter={
                "dstAddr": destination_ip,
                "dstPortFrom": first_dest_port,
                "dstPortTo": last_dest_port,
                "ipProto": ip_protocol,
                "srcAddr": source_ip,
                "srcPortFrom": first_src_port,
                "srcPortTo": last_src_port,
            },
        ),
    )

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class="spanFilterEntry",
            class_config=dict(
                dstAddr=destination_ip,
                dstPortFrom=first_dest_port,
                dstPortTo=last_dest_port,
                ipProto=ip_protocol,
                nameAlias=name_alias,
                srcAddr=source_ip,
                srcPortFrom=first_src_port,
                srcPortTo=last_src_port,
            ),
        )

        aci.get_diff(aci_class="spanFilterEntry")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2022 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

"""
The consolidate filter plugin
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
    name: consolidate
    author: Sagar Paul (@KB-perByte)
    version_added: "2.6.0"
    short_description: Consolidate facts together on common attributes.
    description:
        - This plugin presents collective structured data including all supplied facts grouping on common attributes mentioned.
        - All other boolean parameter defaults to False unless parameters is explicitly mentioned.
        - Using the parameters below- C(data_sources|ansible.utils.consolidate(fail_missing_match_key=False)))
    options:
      data_sources:
        description:
        - This option represents a list of dictionaries to perform the operation on.
        - For example C(facts_source|ansible.utils.consolidate(fail_missing_match_key=False))), in this case C(facts_source) represents this option.
        type: list
        elements: dict
        required: True
        suboptions:
          data:
            description: Specify facts data that gets consolidated.
            type: raw
            required: True
          match_key:
            description: Specify key to match on.
            type: str
            required: True
          name:
            description: Specify the name with which the result set be created.
            type: str
            required: True
      fail_missing_match_key:
        description: Fail if match_key is not found in a specific data set.
        type: bool
        default: True
      fail_missing_match_value:
        description: Fail if the match key's value is not found in every data source.
        type: bool
        default: True
      fail_duplicate:
        description: Fail if the match key's value exists more than once in a given data set.
        type: bool
        default: True
"""

EXAMPLES = r"""

# Consolidated filter plugin example
# ----------------------------------

# play.yml
- name: Define some test data
  ansible.builtin.set_fact:
    values:
      - name: a
        value: 1
      - name: b
        value: 2
      - name: c
        value: 3
    colors:
      - name: a
        color: red
      - name: b
        color: green
      - name: c
        color: blue
- name: Define some test data
  ansible.builtin.set_fact:
    base_data:
      - data: '{{ values }}'
        match_key: name
        name: values
      - data: '{{ colors }}'
        match_key: name
        name: colors
- name: Consolidate the data source using the name key
  ansible.builtin.set_fact:
    consolidated: '{{ data_sources|ansible.utils.consolidate }}'
    vars:
      sizes:
        - name: a
          size: small
        - name: b
          size: medium
        - name: c
          size: large
      additional_data_source:
        - data: '{{ sizes }}'
          match_key: name
          name: sizes
      data_sources: '{{ base_data + additional_data_source }}'


# Output

# ok: [localhost] => {
#     "ansible_facts": {
#         "consolidated": {
#             "a": {
#                 "colors": {
#                     "color": "red",
#                     "name": "a"
#                 },
#                 "sizes": {
#                     "name": "a",
#                     "size": "small"
#                 },
#                 "values": {
#                     "name": "a",
#                     "value": 1
#                 }
#             },
#             "b": {
#                 "colors": {
#                     "color": "green",
#                     "name": "b"
#                 },
#                 "sizes": {
#                     "name": "b",
#                     "size": "medium"
#                 },
#                 "values": {
#                     "name": "b",
#                     "value": 2
#                 }
#             },
#             "c": {
#                 "colors": {
#                     "color": "blue",
#                     "name": "c"
#                 },
#                 "sizes": {
#                     "name": "c",
#                     "size": "large"
#                 },
#                 "values": {
#                     "name": "c",
#                     "value": 3
#                 }
#             }
#         }
#     },
#     "changed": false
# }

- name: Consolidate the data source using different keys
  ansible.builtin.set_fact: null
  consolidated: '{{ data_sources|ansible.utils.consolidate }}'
  vars:
    sizes:
      - title: a
        size: small
      - title: b
        size: medium
      - title: c
        size: large
    additional_data_source:
      - data: '{{ sizes }}'
        match_key: title
        name: sizes
    data_sources: '{{ base_data + additional_data_source }}'


# Output

# ok: [localhost] => {
#     "ansible_facts": {
#         "consolidated": {
#             "a": {
#                 "colors": {
#                     "color": "red",
#                     "name": "a"
#                 },
#                 "sizes": {
#                     "size": "small",
#                     "title": "a"
#                 },
#                 "values": {
#                     "name": "a",
#                     "value": 1
#                 }
#             },
#             "b": {
#                 "colors": {
#                     "color": "green",
#                     "name": "b"
#                 },
#                 "sizes": {
#                     "size": "medium",
#                     "title": "b"
#                 },
#                 "values": {
#                     "name": "b",
#                     "value": 2
#                 }
#             },
#             "c": {
#                 "colors": {
#                     "color": "blue",
#                     "name": "c"
#                 },
#                 "sizes": {
#                     "size": "large",
#                     "title": "c"
#                 },
#                 "values": {
#                     "name": "c",
#                     "value": 3
#                 }
#             }
#         }
#     },
#     "changed": false
# }

- name: Consolidate the data source using the name key (fail_missing_match_key)
  ansible.builtin.set_fact: null
  consolidated: '{{ data_sources|ansible.utils.consolidate(fail_missing_match_key=True) }}'
  ignore_errors: true
  vars:
    sizes:
      - size: small
      - size: medium
      - size: large
    additional_data_source:
      - data: '{{ sizes }}'
        match_key: name
        name: sizes
    data_sources: '{{ base_data + additional_data_source }}'

# Output

# fatal: [localhost]: FAILED! => {
#     "msg": "Error when using plugin 'consolidate': 'fail_missing_match_key'
#                   reported missing match key 'name' in data source 3 in list entry 1,
#                            missing match key 'name' in data source 3 in list entry 2,
#                            missing match key 'name' in data source 3 in list entry 3"
# }

- name: Consolidate the data source using the name key (fail_missing_match_value)
  ansible.builtin.set_fact:
    consolidated: "{{ data_sources|ansible.utils.consolidate(fail_missing_match_value=True) }}"
  ignore_errors: true
  vars:
    sizes:
      - name: a
        size: small
      - name: b
        size: medium
    additional_data_source:
      - data: "{{ sizes }}"
        match_key: name
        name: sizes
    data_sources: "{{ base_data + additional_data_source }}"

# fatal: [localhost]: FAILED! => {
#     "msg": "Error when using plugin 'consolidate': 'fail_missing_match_value'
#                   reported missing match value c in data source 3"
# }

- name: Consolidate the data source using the name key (fail_duplicate)
  ansible.builtin.set_fact:
    consolidated: "{{ data_sources|ansible.utils.consolidate(fail_duplicate=True) }}"
  ignore_errors: true
  vars:
    sizes:
      - name: a
        size: small
      - name: a
        size: small
    additional_data_source:
      - data: "{{ sizes }}"
        match_key: name
        name: sizes
    data_sources: "{{ base_data + additional_data_source }}"

# fatal: [localhost]: FAILED! => {
#     "msg": "Error when using plugin 'consolidate': 'fail_duplicate'
#                   reported duplicate values in data source 3"
# }

# facts.yml

# interfaces:
#   - name: GigabitEthernet0/0
#     enabled: true
#     duplex: auto
#     speed: auto
#     note:
#       - Connected green wire
#   - name: GigabitEthernet0/1
#     description: Configured by Ansible - Interface 1
#     mtu: 1500
#     speed: auto
#     duplex: auto
#     enabled: true
#     note:
#       - Connected blue wire
#       - Configured by Paul
#     vifs:
#       - vlan_id: 100
#         description: Eth1 - VIF 100
#         mtu: 400
#         enabled: true
#         comment: Needs reconfiguration
#       - vlan_id: 101
#         description: Eth1 - VIF 101
#         enabled: true
#   - name: GigabitEthernet0/2
#     description: Configured by Ansible - Interface 2 (ADMIN DOWN)
#     mtu: 600
#     enabled: false
# l2_interfaces:
#   - name: GigabitEthernet0/0
#   - mode: access
#     name: GigabitEthernet0/1
#     trunk:
#       allowed_vlans:
#         - "11"
#         - "12"
#         - "59"
#         - "67"
#         - "75"
#         - "77"
#         - "81"
#         - "100"
#         - 400-408
#         - 411-413
#         - "415"
#         - "418"
#         - "982"
#         - "986"
#         - "988"
#         - "993"
#   - mode: trunk
#     name: GigabitEthernet0/2
#     trunk:
#       allowed_vlans:
#         - "11"
#         - "12"
#         - "59"
#         - "67"
#         - "75"
#         - "77"
#         - "81"
#         - "100"
#         - 400-408
#         - 411-413
#         - "415"
#         - "418"
#         - "982"
#         - "986"
#         - "988"
#         - "993"
#       encapsulation: dot1q
# l3_interfaces:
#   - ipv4:
#       - address: 192.168.0.2/24
#     name: GigabitEthernet0/0
#   - name: GigabitEthernet0/1
#   - name: GigabitEthernet0/2
#   - name: Loopback888
#   - name: Loopback999

# Playbook
- name: Build the facts collection
  set_fact:
    data_sources:
      - data: '{{ interfaces }}'
        match_key: name
        name: interfaces
      - data: '{{ l2_interfaces }}'
        match_key: name
        name: l2_interfaces
      - data: '{{ l3_interfaces }}'
        match_key: name
        name: l3_interfaces
- name: Combine all the facts based on match_keys
  set_fact:
    combined: >-
      {{ data_sources|ansible.utils.consolidate(fail_missing_match_value=False)
      }}

# Output
# ok: [localhost] => {
#     "ansible_facts": {
#         "data_sources": [
#             {
#                 "data": [
#                     {
#                         "duplex": "auto",
#                         "enabled": true,
#                         "name": "GigabitEthernet0/0",
#                         "note": [
#                             "Connected green wire"
#                         ],
#                         "speed": "auto"
#                     },
#                     {
#                         "description": "Configured by Ansible - Interface 1",
#                         "duplex": "auto",
#                         "enabled": true,
#                         "mtu": 1500,
#                         "name": "GigabitEthernet0/1",
#                         "note": [
#                             "Connected blue wire",
#                             "Configured by Paul"
#                         ],
#                         "speed": "auto",
#                         "vifs": [
#                             {
#                                 "comment": "Needs reconfiguration",
#                                 "description": "Eth1 - VIF 100",
#                                 "enabled": true,
#                                 "mtu": 400,
#                                 "vlan_id": 100
#                             },
#                             {
#                                 "description": "Eth1 - VIF 101",
#                                 "enabled": true,
#                                 "vlan_id": 101
#                             }
#                         ]
#                     },
#                     {
#                         "description": "Configured by Ansible - Interface 2 (ADMIN DOWN)",
#                         "enabled": false,
#                         "mtu": 600,
#                         "name": "GigabitEthernet0/2"
#                     }
#                 ],
#                 "match_key": "name",
#                 "name": "interfaces"
#             },
#             {
#                 "data": [
#                     {
#                         "name": "GigabitEthernet0/0"
#                     },
#                     {
#                         "mode": "access",
#                         "name": "GigabitEthernet0/1",
#                         "trunk": {
#                             "allowed_vlans": [
#                                 "11",
#                                 "12",
#                                 "59",
#                                 "67",
#                                 "75",
#                                 "77",
#                                 "81",
#                                 "100",
#                                 "400-408",
#                                 "411-413",
#                                 "415",
#                                 "418",
#                                 "982",
#                                 "986",
#                                 "988",
#                                 "993"
#                             ]
#                         }
#                     },
#                     {
#                         "mode": "trunk",
#                         "name": "GigabitEthernet0/2",
#                         "trunk": {
#                             "allowed_vlans": [
#                                 "11",
#                                 "12",
#                                 "59",
#                                 "67",
#                                 "75",
#                                 "77",
#                                 "81",
#                                 "100",
#                                 "400-408",
#                                 "411-413",
#                                 "415",
#                                 "418",
#                                 "982",
#                                 "986",
#                                 "988",
#                                 "993"
#                             ],
#                             "encapsulation": "dot1q"
#                         }
#                     }
#                 ],
#                 "match_key": "name",
#                 "name": "l2_interfaces"
#             },
#             {
#                 "data": [
#                     {
#                         "ipv4": [
#                             {
#                                 "address": "192.168.0.2/24"
#                             }
#                         ],
#                         "name": "GigabitEthernet0/0"
#                     },
#                     {
#                         "name": "GigabitEthernet0/1"
#                     },
#                     {
#                         "name": "GigabitEthernet0/2"
#                     },
#                     {
#                         "name": "Loopback888"
#                     },
#                     {
#                         "name": "Loopback999"
#                     }
#                 ],
#                 "match_key": "name",
#                 "name": "l3_interfaces"
#             }
#         ]
#     },
#     "changed": false
# }
# Read vars_file 'facts.yml'

# TASK [Combine all the facts based on match_keys]
# ok: [localhost] => {
#     "ansible_facts": {
#         "combined": {
#             "GigabitEthernet0/0": {
#                 "interfaces": {
#                     "duplex": "auto",
#                     "enabled": true,
#                     "name": "GigabitEthernet0/0",
#                     "note": [
#                         "Connected green wire"
#                     ],
#                     "speed": "auto"
#                 },
#                 "l2_interfaces": {
#                     "name": "GigabitEthernet0/0"
#                 },
#                 "l3_interfaces": {
#                     "ipv4": [
#                         {
#                             "address": "192.168.0.2/24"
#                         }
#                     ],
#                     "name": "GigabitEthernet0/0"
#                 }
#             },
#             "GigabitEthernet0/1": {
#                 "interfaces": {
#                     "description": "Configured by Ansible - Interface 1",
#                     "duplex": "auto",
#                     "enabled": true,
#                     "mtu": 1500,
#                     "name": "GigabitEthernet0/1",
#                     "note": [
#                         "Connected blue wire",
#                         "Configured by Paul"
#                     ],
#                     "speed": "auto",
#                     "vifs": [
#                         {
#                             "comment": "Needs reconfiguration",
#                             "description": "Eth1 - VIF 100",
#                             "enabled": true,
#                             "mtu": 400,
#                             "vlan_id": 100
#                         },
#                         {
#                             "description": "Eth1 - VIF 101",
#                             "enabled": true,
#                             "vlan_id": 101
#                         }
#                     ]
#                 },
#                 "l2_interfaces": {
#                     "mode": "access",
#                     "name": "GigabitEthernet0/1",
#                     "trunk": {
#                         "allowed_vlans": [
#                             "11",
#                             "12",
#                             "59",
#                             "67",
#                             "75",
#                             "77",
#                             "81",
#                             "100",
#                             "400-408",
#                             "411-413",
#                             "415",
#                             "418",
#                             "982",
#                             "986",
#                             "988",
#                             "993"
#                         ]
#                     }
#                 },
#                 "l3_interfaces": {
#                     "name": "GigabitEthernet0/1"
#                 }
#             },
#             "GigabitEthernet0/2": {
#                 "interfaces": {
#                     "description": "Configured by Ansible - Interface 2 (ADMIN DOWN)",
#                     "enabled": false,
#                     "mtu": 600,
#                     "name": "GigabitEthernet0/2"
#                 },
#                 "l2_interfaces": {
#                     "mode": "trunk",
#                     "name": "GigabitEthernet0/2",
#                     "trunk": {
#                         "allowed_vlans": [
#                             "11",
#                             "12",
#                             "59",
#                             "67",
#                             "75",
#                             "77",
#                             "81",
#                             "100",
#                             "400-408",
#                             "411-413",
#                             "415",
#                             "418",
#                             "982",
#                             "986",
#                             "988",
#                             "993"
#                         ],
#                         "encapsulation": "dot1q"
#                     }
#                 },
#                 "l3_interfaces": {
#                     "name": "GigabitEthernet0/2"
#                 }
#             },
#             "Loopback888": {
#                 "interfaces": {},
#                 "l2_interfaces": {},
#                 "l3_interfaces": {
#                     "name": "Loopback888"
#                 }
#             },
#             "Loopback999": {
#                 "interfaces": {},
#                 "l2_interfaces": {},
#                 "l3_interfaces": {
#                     "name": "Loopback999"
#                 }
#             }
#         }
#     },
#     "changed": false
# }

# Failing on missing match values
# -------------------------------

# facts.yaml
# interfaces:
#   - name: GigabitEthernet0/0
#     enabled: true
#     duplex: auto
#     speed: auto
#     note:
#       - Connected green wire
#   - name: GigabitEthernet0/1
#     description: Configured by Ansible - Interface 1
#     mtu: 1500
#     speed: auto
#     duplex: auto
#     enabled: true
#     note:
#       - Connected blue wire
#       - Configured by Paul
#     vifs:
#       - vlan_id: 100
#         description: Eth1 - VIF 100
#         mtu: 400
#         enabled: true
#         comment: Needs reconfiguration
#       - vlan_id: 101
#         description: Eth1 - VIF 101
#         enabled: true
#   - name: GigabitEthernet0/2
#     description: Configured by Ansible - Interface 2 (ADMIN DOWN)
#     mtu: 600
#     enabled: false
# l2_interfaces:
#   - name: GigabitEthernet0/0
#   - mode: access
#     name: GigabitEthernet0/1
#     trunk:
#       allowed_vlans:
#         - "11"
#         - "12"
#         - "59"
#         - "67"
#         - "75"
#         - "77"
#         - "81"
#         - "100"
#         - 400-408
#         - 411-413
#         - "415"
#         - "418"
#         - "982"
#         - "986"
#         - "988"
#         - "993"
#   - mode: trunk
#     name: GigabitEthernet0/2
#     trunk:
#       allowed_vlans:
#         - "11"
#         - "12"
#         - "59"
#         - "67"
#         - "75"
#         - "77"
#         - "81"
#         - "100"
#         - 400-408
#         - 411-413
#         - "415"
#         - "418"
#         - "982"
#         - "986"
#         - "988"
#         - "993"
#       encapsulation: dot1q
# l3_interfaces:
#   - ipv4:
#       - address: 192.168.0.2/24
#     name: GigabitEthernet0/0
#   - name: GigabitEthernet0/1
#   - name: GigabitEthernet0/2
#   - name: Loopback888
#   - name: Loopback999

# Playbook
- name: Build the facts collection
  set_fact:
    data_sources:
      - data: '{{ interfaces }}'
        match_key: name
        name: interfaces
      - data: '{{ l2_interfaces }}'
        match_key: name
        name: l2_interfaces
      - data: '{{ l3_interfaces }}'
        match_key: name
        name: l3_interfaces
- name: Combine all the facts based on match_keys
  set_fact:
    combined: >-
      {{ data_sources|ansible.utils.consolidate(fail_missing_match_value=True)
      }}

# Output
# ok: [localhost] => {
#     "ansible_facts": {
#         "data_sources": [
#             {
#                 "data": [
#                     {
#                         "duplex": "auto",
#                         "enabled": true,
#                         "name": "GigabitEthernet0/0",
#                         "note": [
#                             "Connected green wire"
#                         ],
#                         "speed": "auto"
#                     },
#                     {
#                         "description": "Configured by Ansible - Interface 1",
#                         "duplex": "auto",
#                         "enabled": true,
#                         "mtu": 1500,
#                         "name": "GigabitEthernet0/1",
#                         "note": [
#                             "Connected blue wire",
#                             "Configured by Paul"
#                         ],
#                         "speed": "auto",
#                         "vifs": [
#                             {
#                                 "comment": "Needs reconfiguration",
#                                 "description": "Eth1 - VIF 100",
#                                 "enabled": true,
#                                 "mtu": 400,
#                                 "vlan_id": 100
#                             },
#                             {
#                                 "description": "Eth1 - VIF 101",
#                                 "enabled": true,
#                                 "vlan_id": 101
#                             }
#                         ]
#                     },
#                     {
#                         "description": "Configured by Ansible - Interface 2 (ADMIN DOWN)",
#                         "enabled": false,
#                         "mtu": 600,
#                         "name": "GigabitEthernet0/2"
#                     }
#                 ],
#                 "match_key": "name",
#                 "name": "interfaces"
#             },
#             {
#                 "data": [
#                     {
#                         "name": "GigabitEthernet0/0"
#                     },
#                     {
#                         "mode": "access",
#                         "name": "GigabitEthernet0/1",
#                         "trunk": {
#                             "allowed_vlans": [
#                                 "11",
#                                 "12",
#                                 "59",
#                                 "67",
#                                 "75",
#                                 "77",
#                                 "81",
#                                 "100",
#                                 "400-408",
#                                 "411-413",
#                                 "415",
#                                 "418",
#                                 "982",
#                                 "986",
#                                 "988",
#                                 "993"
#                             ]
#                         }
#                     },
#                     {
#                         "mode": "trunk",
#                         "name": "GigabitEthernet0/2",
#                         "trunk": {
#                             "allowed_vlans": [
#                                 "11",
#                                 "12",
#                                 "59",
#                                 "67",
#                                 "75",
#                                 "77",
#                                 "81",
#                                 "100",
#                                 "400-408",
#                                 "411-413",
#                                 "415",
#                                 "418",
#                                 "982",
#                                 "986",
#                                 "988",
#                                 "993"
#                             ],
#                             "encapsulation": "dot1q"
#                         }
#                     }
#                 ],
#                 "match_key": "name",
#                 "name": "l2_interfaces"
#             },
#             {
#                 "data": [
#                     {
#                         "ipv4": [
#                             {
#                                 "address": "192.168.0.2/24"
#                             }
#                         ],
#                         "name": "GigabitEthernet0/0"
#                     },
#                     {
#                         "name": "GigabitEthernet0/1"
#                     },
#                     {
#                         "name": "GigabitEthernet0/2"
#                     },
#                     {
#                         "name": "Loopback888"
#                     },
#                     {
#                         "name": "Loopback999"
#                     }
#                 ],
#                 "match_key": "name",
#                 "name": "l3_interfaces"
#             }
#         ]
#     },
#     "changed": false
# }
# Read vars_file 'facts.yml'

# TASK [Combine all the facts based on match_keys]
# fatal: [localhost]: FAILED! => {
#     "msg": "Error when using plugin 'consolidate': 'fail_missing_match_value' reported Missing match value Loopback999,
#     Loopback888 in data source 0, Missing match value Loopback999, Loopback888 in data source 1"
# }

# Failing on missing match keys
# -----------------------------

# facts.yaml
# interfaces:
#   - name: GigabitEthernet0/0
#     enabled: true
#     duplex: auto
#     speed: auto
#     note:
#       - Connected green wire
#   - name: GigabitEthernet0/1
#     description: Configured by Ansible - Interface 1
#     mtu: 1500
#     speed: auto
#     duplex: auto
#     enabled: true
#     note:
#       - Connected blue wire
#       - Configured by Paul
#     vifs:
#       - vlan_id: 100
#         description: Eth1 - VIF 100
#         mtu: 400
#         enabled: true
#         comment: Needs reconfiguration
#       - vlan_id: 101
#         description: Eth1 - VIF 101
#         enabled: true
#   - name: GigabitEthernet0/2
#     description: Configured by Ansible - Interface 2 (ADMIN DOWN)
#     mtu: 600
#     enabled: false
# l2_interfaces:
#   - name: GigabitEthernet0/0
#   - mode: access
#     name: GigabitEthernet0/1
#     trunk:
#       allowed_vlans:
#         - "11"
#         - "12"
#         - "59"
#         - "67"
#         - "75"
#         - "77"
#         - "81"
#         - "100"
#         - 400-408
#         - 411-413
#         - "415"
#         - "418"
#         - "982"
#         - "986"
#         - "988"
#         - "993"
#   - mode: trunk
#     name: GigabitEthernet0/2
#     trunk:
#       allowed_vlans:
#         - "11"
#         - "12"
#         - "59"
#         - "67"
#         - "75"
#         - "77"
#         - "81"
#         - "100"
#         - 400-408
#         - 411-413
#         - "415"
#         - "418"
#         - "982"
#         - "986"
#         - "988"
#         - "993"
#       encapsulation: dot1q
# l3_interfaces:
#   - ipv4:
#       - address: 192.168.0.2/24
#     inft_name: GigabitEthernet0/0
#   - inft_name: GigabitEthernet0/1
#   - inft_name: GigabitEthernet0/2
#   - inft_name: Loopback888
#   - inft_name: Loopback999

# Playbook
- name: Build the facts collection
  set_fact:
    data_sources:
      - data: '{{ interfaces }}'
        match_key: name
        name: interfaces
      - data: '{{ l2_interfaces }}'
        match_key: name
        name: l2_interfaces
      - data: '{{ l3_interfaces }}'
        match_key: name
        name: l3_interfaces
- name: Combine all the facts based on match_keys
  set_fact:
    combined: '{{ data_sources|ansible.utils.consolidate(fail_missing_match_key=True) }}'

# Output
# ok: [localhost] => {
#     "ansible_facts": {
#         "data_sources": [
#             {
#                 "data": [
#                     {
#                         "duplex": "auto",
#                         "enabled": true,
#                         "name": "GigabitEthernet0/0",
#                         "note": [
#                             "Connected green wire"
#                         ],
#                         "speed": "auto"
#                     },
#                     {
#                         "description": "Configured by Ansible - Interface 1",
#                         "duplex": "auto",
#                         "enabled": true,
#                         "mtu": 1500,
#                         "name": "GigabitEthernet0/1",
#                         "note": [
#                             "Connected blue wire",
#                             "Configured by Paul"
#                         ],
#                         "speed": "auto",
#                         "vifs": [
#                             {
#                                 "comment": "Needs reconfiguration",
#                                 "description": "Eth1 - VIF 100",
#                                 "enabled": true,
#                                 "mtu": 400,
#                                 "vlan_id": 100
#                             },
#                             {
#                                 "description": "Eth1 - VIF 101",
#                                 "enabled": true,
#                                 "vlan_id": 101
#                             }
#                         ]
#                     },
#                     {
#                         "description": "Configured by Ansible - Interface 2 (ADMIN DOWN)",
#                         "enabled": false,
#                         "mtu": 600,
#                         "name": "GigabitEthernet0/2"
#                     }
#                 ],
#                 "match_key": "name",
#                 "name": "interfaces"
#             },
#             {
#                 "data": [
#                     {
#                         "name": "GigabitEthernet0/0"
#                     },
#                     {
#                         "mode": "access",
#                         "name": "GigabitEthernet0/1",
#                         "trunk": {
#                             "allowed_vlans": [
#                                 "11",
#                                 "12",
#                                 "59",
#                                 "67",
#                                 "75",
#                                 "77",
#                                 "81",
#                                 "100",
#                                 "400-408",
#                                 "411-413",
#                                 "415",
#                                 "418",
#                                 "982",
#                                 "986",
#                                 "988",
#                                 "993"
#                             ]
#                         }
#                     },
#                     {
#                         "mode": "trunk",
#                         "name": "GigabitEthernet0/2",
#                         "trunk": {
#                             "allowed_vlans": [
#                                 "11",
#                                 "12",
#                                 "59",
#                                 "67",
#                                 "75",
#                                 "77",
#                                 "81",
#                                 "100",
#                                 "400-408",
#                                 "411-413",
#                                 "415",
#                                 "418",
#                                 "982",
#                                 "986",
#                                 "988",
#                                 "993"
#                             ],
#                             "encapsulation": "dot1q"
#                         }
#                     }
#                 ],
#                 "match_key": "name",
#                 "name": "l2_interfaces"
#             },
#             {
#                 "data": [
#                     {
#                         "inft_name": "GigabitEthernet0/0",
#                         "ipv4": [
#                             {
#                                 "address": "192.168.0.2/24"
#                             }
#                         ]
#                     },
#                     {
#                         "inft_name": "GigabitEthernet0/1"
#                     },
#                     {
#                         "inft_name": "GigabitEthernet0/2"
#                     },
#                     {
#                         "inft_name": "Loopback888"
#                     },
#                     {
#                         "inft_name": "Loopback999"
#                     }
#                 ],
#                 "match_key": "name",
#                 "name": "l3_interfaces"
#             }
#         ]
#     },
#     "changed": false
# }
# Read vars_file 'facts.yml'

# TASK [Combine all the facts based on match_keys]
# fatal: [localhost]: FAILED! => {
#     "msg": "Error when using plugin 'consolidate': 'fail_missing_match_key' reported Missing match
#     key 'name' in data source 2 in list entry 0, Missing match key 'name' in data
#     source 2 in list entry 1, Missing match key 'name' in data source 2 in list
#     entry 2, Missing match key 'name' in data source 2 in list entry 3, Missing
#     match key 'name' in data source 2 in list entry 4"
# }

# Failing on duplicate values in facts
# ------------------------------------

# facts.yaml
# interfaces:
#   - name: GigabitEthernet0/0
#     enabled: true
#     duplex: auto
#     speed: auto
#     note:
#       - Connected green wire
#   - name: GigabitEthernet0/1
#     description: Configured by Ansible - Interface 1
#     mtu: 1500
#     speed: auto
#     duplex: auto
#     enabled: true
#     note:
#       - Connected blue wire
#       - Configured by Paul
#     vifs:
#       - vlan_id: 100
#         description: Eth1 - VIF 100
#         mtu: 400
#         enabled: true
#         comment: Needs reconfiguration
#       - vlan_id: 101
#         description: Eth1 - VIF 101
#         enabled: true
#   - name: GigabitEthernet0/2
#     description: Configured by Ansible - Interface 2 (ADMIN DOWN)
#     mtu: 600
#     enabled: false
# l2_interfaces:
#   - name: GigabitEthernet0/0
#   - name: GigabitEthernet0/0
#   - mode: access
#     name: GigabitEthernet0/1
#     trunk:
#       allowed_vlans:
#         - "11"
#         - "12"
#         - "59"
#         - "67"
#         - "75"
#         - "77"
#         - "81"
#         - "100"
#         - 400-408
#         - 411-413
#         - "415"
#         - "418"
#         - "982"
#         - "986"
#         - "988"
#         - "993"
#   - mode: trunk
#     name: GigabitEthernet0/2
#     trunk:
#       allowed_vlans:
#         - "11"
#         - "12"
#         - "59"
#         - "67"
#         - "75"
#         - "77"
#         - "81"
#         - "100"
#         - 400-408
#         - 411-413
#         - "415"
#         - "418"
#         - "982"
#         - "986"
#         - "988"
#         - "993"
#       encapsulation: dot1q
# l3_interfaces:
#   - ipv4:
#       - address: 192.168.0.2/24
#     name: GigabitEthernet0/0
#   - name: GigabitEthernet0/1
#   - name: GigabitEthernet0/2
#   - name: Loopback888
#   - name: Loopback999

# Playbook
- name: Build the facts collection
  set_fact:
    data_sources:
      - data: '{{ interfaces }}'
        match_key: name
        name: interfaces
      - data: '{{ l2_interfaces }}'
        match_key: name
        name: l2_interfaces
      - data: '{{ l3_interfaces }}'
        match_key: name
        name: l3_interfaces
- name: Combine all the facts based on match_keys
  set_fact:
    combined: '{{ data_sources|ansible.utils.consolidate(fail_duplicate=True) }}'

# Output
# ok: [localhost] => {
#     "ansible_facts": {
#         "data_sources": [
#             {
#                 "data": [
#                     {
#                         "duplex": "auto",
#                         "enabled": true,
#                         "name": "GigabitEthernet0/0",
#                         "note": [
#                             "Connected green wire"
#                         ],
#                         "speed": "auto"
#                     },
#                     {
#                         "description": "Configured by Ansible - Interface 1",
#                         "duplex": "auto",
#                         "enabled": true,
#                         "mtu": 1500,
#                         "name": "GigabitEthernet0/1",
#                         "note": [
#                             "Connected blue wire",
#                             "Configured by Paul"
#                         ],
#                         "speed": "auto",
#                         "vifs": [
#                             {
#                                 "comment": "Needs reconfiguration",
#                                 "description": "Eth1 - VIF 100",
#                                 "enabled": true,
#                                 "mtu": 400,
#                                 "vlan_id": 100
#                             },
#                             {
#                                 "description": "Eth1 - VIF 101",
#                                 "enabled": true,
#                                 "vlan_id": 101
#                             }
#                         ]
#                     },
#                     {
#                         "description": "Configured by Ansible - Interface 2 (ADMIN DOWN)",
#                         "enabled": false,
#                         "mtu": 600,
#                         "name": "GigabitEthernet0/2"
#                     }
#                 ],
#                 "match_key": "name",
#                 "name": "interfaces"
#             },
#             {
#                 "data": [
#                     {
#                         "name": "GigabitEthernet0/0"
#                     },
#                     {
#                         "name": "GigabitEthernet0/0"
#                     },
#                     {
#                         "mode": "access",
#                         "name": "GigabitEthernet0/1",
#                         "trunk": {
#                             "allowed_vlans": [
#                                 "11",
#                                 "12",
#                                 "59",
#                                 "67",
#                                 "75",
#                                 "77",
#                                 "81",
#                                 "100",
#                                 "400-408",
#                                 "411-413",
#                                 "415",
#                                 "418",
#                                 "982",
#                                 "986",
#                                 "988",
#                                 "993"
#                             ]
#                         }
#                     },
#                     {
#                         "mode": "trunk",
#                         "name": "GigabitEthernet0/2",
#                         "trunk": {
#                             "allowed_vlans": [
#                                 "11",
#                                 "12",
#                                 "59",
#                                 "67",
#                                 "75",
#                                 "77",
#                                 "81",
#                                 "100",
#                                 "400-408",
#                                 "411-413",
#                                 "415",
#                                 "418",
#                                 "982",
#                                 "986",
#                                 "988",
#                                 "993"
#                             ],
#                             "encapsulation": "dot1q"
#                         }
#                     }
#                 ],
#                 "match_key": "name",
#                 "name": "l2_interfaces"
#             },
#             {
#                 "data": [
#                     {
#                         "ipv4": [
#                             {
#                                 "address": "192.168.0.2/24"
#                             }
#                         ],
#                         "name": "GigabitEthernet0/0"
#                     },
#                     {
#                         "name": "GigabitEthernet0/1"
#                     },
#                     {
#                         "name": "GigabitEthernet0/2"
#                     },
#                     {
#                         "name": "Loopback888"
#                     },
#                     {
#                         "name": "Loopback999"
#                     }
#                 ],
#                 "match_key": "name",
#                 "name": "l3_interfaces"
#             }
#         ]
#     },
#     "changed": false
# }
# Read vars_file 'facts.yml'

# TASK [Combine all the facts based on match_keys]
# fatal: [localhost]: FAILED! => {
#     "msg": "Error when using plugin 'consolidate': 'fail_duplicate' reported Duplicate values in data source 1"
# }
"""

from ansible.errors import AnsibleFilterError

from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
    AnsibleArgSpecValidator,
)
from ansible_collections.ansible.utils.plugins.plugin_utils.consolidate import consolidate


try:
    from jinja2.filters import pass_environment
except ImportError:
    from jinja2.filters import environmentfilter as pass_environment


@pass_environment
def _consolidate(*args, **kwargs):
    """Consolidate facts together on common attributes"""

    keys = [
        "data_sources",
        "fail_missing_match_key",
        "fail_missing_match_value",
        "fail_duplicate",
    ]
    data = dict(zip(keys, args[1:]))
    data.update(kwargs)
    aav = AnsibleArgSpecValidator(data=data, schema=DOCUMENTATION, name="consolidate")
    valid, errors, updated_data = aav.validate()
    if not valid:
        raise AnsibleFilterError(errors)
    return consolidate(**updated_data)


class FilterModule(object):
    """Consolidate"""

    def filters(self):
        """A mapping of filter names to functions"""
        return {"consolidate": _consolidate}



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: duo_identity_sync_status
short_description: Resource module for Duo Identity Sync Status
description:
- Manage operation update of the resource Duo Identity Sync Status.
- Duo-identitysync - update sync status.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  errorList:
    description: Duo Identity Sync Status's errorList.
    elements: dict
    suboptions:
      reason:
        description: Reason user failed sync.
        type: str
      user:
        description: User to be synced to Duo.
        suboptions:
          directoryname:
            description: Active directory that duo user is contained in.
            type: str
          email:
            description: Email of Duo user.
            type: str
          firstname:
            description: First name of Duo user.
            type: str
          groupname:
            description: Acrive directory group that duo user is contained in.
            type: str
          lastname:
            description: Last name of duo user.
            type: str
          notes:
            description: Notes of Duo user.
            type: str
          realname:
            description: Realname of Duo user.
            type: str
          status:
            description: Status of Duo user.
            type: str
          username:
            description: Username of Duo user.
            type: str
        type: dict
    type: list
  status:
    description: Status of sync.
    type: str
  syncName:
    description: SyncName path parameter. Sync connection to be updated.
    type: str
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
notes:
  - SDK Method used are
    duo_identity_sync.DuoIdentitySync.update_status,

  - Paths used are
    put /api/v1/duo-identitysync/identitysync/status/{syncName},

"""

EXAMPLES = r"""
- name: Update by name
  cisco.ise.duo_identity_sync_status:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    errorList:
    - reason: string
      user:
        directoryname: string
        email: string
        firstname: string
        groupname: string
        lastname: string
        notes: string
        realname: string
        status: string
        username: string
    status: string
    syncName: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: str
  sample: >
    "'string'"
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2023, Anvitha Jain (@anvjain) <anvjain@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: aci_l3out_bfd_multihop_interface_profile
short_description: Manage BFD Multihop Interface profiles (bfd:MhIfP)
description:
- Manage BFD Multihop Interface profile configuration on Cisco ACI fabrics
- Only available in APIC version 5.2 or later and for non-cloud APICs
options:
  tenant:
    description:
    - Name of an existing tenant
    type: str
    aliases: [ tenant_name ]
  l3out:
    description:
    - Name of an existing L3Out
    type: str
    aliases: [ l3out_name ]
  l3out_logical_node_profile:
    description:
    - Name of an existing L3Out Logical Node profile
    type: str
    aliases: [ logical_node_profile, logical_node_profile_name ]
  l3out_logical_interface_profile:
    description:
    - Name of an existing L3Out Logical Interface profile
    type: str
    aliases: [ logical_interface_profile, logical_interface_profile_name ]
  name:
    description:
    - Name of the BFD Multihop Interface Profile object
    type: str
    aliases: [ bfd_multihop_interface_profile ]
  name_alias:
    description:
    - Name Alias of the BFD Multihop Interface Profile object
    type: str
  description:
    description:
    - Description of the BFD Multihop Interface Profile object
    type: str
    aliases: [ descr ]
  authentication_type:
    description:
    - Authentication Type of the BFD Multihop Interface Profile object
    - APIC sets the default value to none.
    type: str
    choices: [ none, sha1 ]
  key:
    description:
    - Authentication Key of the BFD Multihop Interface Profile object.
    - Providing this option will always result in a change because it is a secure property that cannot be retrieved from APIC.
    type: str
  key_id:
    description:
    - Authentication Key ID of the BFD Multihop Interface Profile object
    - APIC sets the default value to 3
    - Allowed range is 1-255
    type: int
  bfd_multihop_interface_policy:
    description:
    - The name of the BFD Multihop Interface policy
    type: str
    aliases: [ multihop_interface_policy, multihop_interface_policy_name ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing
    - Use C(query) for listing an object or multiple objects
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation

notes:
- The C(tenant), c(l3out), C(l3out_logical_node_profile) and C(l3out_logical_interface_profile) must exist before using this module in your playbook
  The M(cisco.aci.aci_tenant) modules can be used for this
seealso:
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(bfd:MhIfP)
  link: https://developer.cisco.com/docs/apic-mim-ref/
- module: cisco.aci.aci_tenant
- module: cisco.aci.aci_l3out
- module: cisco.aci.aci_l3out_logical_node_profile
- module: cisco.aci.aci_l3out_logical_interface_profile
- module: cisco.aci.aci_interface_policy_bfd_multihop
author:
- Anvitha Jain (@anvjain)
"""

EXAMPLES = r"""
- name: Add a new L3Out BFD Multihop Interface Profile
  cisco.aci.aci_l3out_bfd_multihop_interface_profile:
    username: admin
    password: SomeSecretPassword
    tenant: ansible_tenant
    l3out: ansible_l3out
    l3out_logical_node_profile: ansible_node_profile
    l3out_logical_interface_profile: ansible_interface_profile
    state: present
  delegate_to: localhost

- name: Query a new L3Out BFD Multihop Interface Profile
  cisco.aci.aci_l3out_bfd_multihop_interface_profile:
    username: admin
    password: SomeSecretPassword
    tenant: ansible_tenant
    l3out: ansible_l3out
    l3out_logical_node_profile: ansible_node_profile
    l3out_logical_interface_profile: ansible_interface_profile
    state: query
  delegate_to: localhost

- name: Query all L3Out BFD Multihop Interface Profile
  cisco.aci.aci_l3out_bfd_multihop_interface_profile:
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost

- name: Delete L3Out BFD Multihop Interface Profile
  cisco.aci.aci_l3out_bfd_multihop_interface_profile:
    username: admin
    password: SomeSecretPassword
    tenant: ansible_tenant
    l3out: ansible_l3out
    l3out_logical_node_profile: ansible_node_profile
    l3out_logical_interface_profile: ansible_interface_profile
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
   current:
     description: The existing configuration from the APIC after the module has finished
     returned: success
     type: list
     sample:
       [
           {
               "fvTenant": {
                   "attributes": {
                       "descr": "Production environment",
                       "dn": "uni/tn-production",
                       "name": "production",
                       "nameAlias": "",
                       "ownerKey": "",
                       "ownerTag": ""
                   }
               }
           }
       ]
   error:
     description: The error information as returned from the APIC
     returned: failure
     type: dict
     sample:
       {
           "code": "122",
           "text": "unknown managed object class foo"
       }
   raw:
     description: The raw output returned by the APIC REST API (xml or json)
     returned: parse error
     type: str
     sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class "/></imdata>'
   sent:
     description: The actual/minimal configuration pushed to the APIC
     returned: info
     type: list
     sample:
       {
           "fvTenant": {
               "attributes": {
                   "descr": "Production environment"
               }
           }
       }
   previous:
     description: The original configuration from the APIC before the module has started
     returned: info
     type: list
     sample:
       [
           {
               "fvTenant": {
                   "attributes": {
                       "descr": "Production",
                       "dn": "uni/tn-production",
                       "name": "production",
                       "nameAlias": "",
                       "ownerKey": "",
                       "ownerTag": ""
                   }
               }
           }
       ]
   proposed:
     description: The assembled configuration from the user-provided parameters
     returned: info
     type: dict
     sample:
       {
           "fvTenant": {
               "attributes": {
                   "descr": "Production environment",
                   "name": "production"
               }
           }
       }
   filter_string:
     description: The filter string used for the request
     returned: failure or debug
     type: str
     sample: ?rsp-prop-include=config-only
   method:
     description: The HTTP method used for the request to the APIC
     returned: failure or debug
     type: str
     sample: POST
   response:
     description: The HTTP response from the APIC
     returned: failure or debug
     type: str
     sample: OK (30 bytes)
   status:
     description: The HTTP status from the APIC
     returned: failure or debug
     type: int
     sample: 200
   url:
     description: The HTTP url used for the request to the APIC
     returned: failure or debug
     type: str
     sample: https://10.11.12.13/api/mo/uni/tn-production.json
   """

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        tenant=dict(type="str", aliases=["tenant_name"]),
        l3out=dict(type="str", aliases=["l3out_name"]),
        l3out_logical_node_profile=dict(type="str", aliases=["logical_node_profile_name", "logical_node_profile"]),
        l3out_logical_interface_profile=dict(type="str", aliases=["logical_interface_profile_name", "logical_interface_profile"]),
        name=dict(type="str", aliases=["bfd_multihop_interface_profile"]),
        name_alias=dict(type="str"),
        description=dict(type="str", aliases=["descr"]),
        authentication_type=dict(type="str", choices=["none", "sha1"]),
        key=dict(type="str", no_log=True),
        key_id=dict(type="int"),
        bfd_multihop_interface_policy=dict(type="str", aliases=["multihop_interface_policy", "multihop_interface_policy_name"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["tenant", "l3out", "l3out_logical_node_profile", "l3out_logical_interface_profile"]],
            ["state", "present", ["tenant", "l3out", "l3out_logical_node_profile", "l3out_logical_interface_profile", "bfd_multihop_interface_policy"]],
        ],
    )

    tenant = module.params.get("tenant")
    l3out = module.params.get("l3out")
    l3out_logical_node_profile = module.params.get("l3out_logical_node_profile")
    l3out_logical_interface_profile = module.params.get("l3out_logical_interface_profile")
    name = module.params.get("name")
    name_alias = module.params.get("name_alias")
    description = module.params.get("description")
    authentication_type = module.params.get("authentication_type")
    key = module.params.get("key")
    key_id = module.params.get("key_id")
    bfd_multihop_interface_policy = module.params.get("bfd_multihop_interface_policy")
    state = module.params.get("state")

    aci = ACIModule(module)
    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="l3extOut",
            aci_rn="out-{0}".format(l3out),
            module_object=l3out,
            target_filter={"name": l3out},
        ),
        subclass_2=dict(
            aci_class="l3extLNodeP",
            aci_rn="lnodep-{0}".format(l3out_logical_node_profile),
            module_object=l3out_logical_node_profile,
            target_filter={"name": l3out_logical_node_profile},
        ),
        subclass_3=dict(
            aci_class="l3extLIfP",
            aci_rn="lifp-{0}".format(l3out_logical_interface_profile),
            module_object=l3out_logical_interface_profile,
            target_filter={"name": l3out_logical_interface_profile},
        ),
        subclass_4=dict(
            aci_class="bfdMhIfP",
            aci_rn="bfdMhIfP",
            module_object="bfdMhIfP",
            target_filter={"name": name},
        ),
        child_classes=["bfdRsMhIfPol"],
    )

    aci.get_existing()
    aci.stdout = str(aci.get_existing())
    if state == "present":
        child_configs = []
        class_config = dict(
            name=name,
            nameAlias=name_alias,
            descr=description,
            type=authentication_type,
            key=key,
        )

        if key_id and key_id not in range(1, 255):
            module.fail_json(msg='The "key_id" must be a value between 1 and 255')
        else:
            class_config["keyId"] = key_id

        if bfd_multihop_interface_policy is not None:
            child_configs.append(dict(bfdRsMhIfPol=dict(attributes=dict(tnBfdMhIfPolName=bfd_multihop_interface_policy))))

        aci.payload(
            aci_class="bfdMhIfP",
            class_config=class_config,
            child_configs=child_configs,
        )

        aci.get_diff(aci_class="bfdMhIfP")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: wireless_controllers_wireless_mobility_groups_mobility_reset
short_description: Resource module for Wireless Controllers Wireless Mobility Groups Mobility Reset
description:
- This module represents an alias of the module wireless_controllers_wireless_mobility_groups_mobility_reset_v1
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  networkDeviceId:
    description: Network device Id of Cisco wireless controller.Obtain the network device
      ID value by using the API call GET - /dna/intent/api/v1/network-device/ip-address/${ipAddress}.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Wireless MobilityResetV1
  description: Complete reference of the MobilityResetV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!mobility-reset
notes:
  - SDK Method used are
    wireless.Wireless.mobility_reset_v1,

  - Paths used are
    post /dna/intent/api/v1/wirelessControllers/wirelessMobilityGroups/mobilityReset,
  - It should be noted that this module is an alias of wireless_controllers_wireless_mobility_groups_mobility_reset_v1

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.wireless_controllers_wireless_mobility_groups_mobility_reset:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    networkDeviceId: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The iosxr lag_interfaces fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type


import re

from copy import deepcopy

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.argspec.lag_interfaces.lag_interfaces import (
    Lag_interfacesArgs,
)


class Lag_interfacesFacts(object):
    """The iosxr lag_interfaces fact class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = Lag_interfacesArgs.argument_spec
        spec = deepcopy(self.argument_spec)
        if subspec:
            if options:
                facts_argument_spec = spec[subspec][options]
            else:
                facts_argument_spec = spec[subspec]
        else:
            facts_argument_spec = spec

        self.generated_spec = utils.generate_dict(facts_argument_spec)

    def get_config(self, connection):
        return connection.get_config(flags="interface")

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for lag_interfaces
        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf
        :rtype: dictionary
        :returns: facts
        """

        if not data:
            data = self.get_config(connection)
        interfaces = ("\n" + data).split("\ninterface ")

        objs = []

        for interface in interfaces:
            if interface.startswith("Bundle-Ether"):
                obj = self.render_config(
                    self.generated_spec,
                    interface,
                    interfaces,
                )
                if obj:
                    objs.append(obj)

        ansible_facts["ansible_network_resources"].pop("lag_interfaces", None)
        facts = {}

        facts["lag_interfaces"] = []
        params = utils.validate_config(self.argument_spec, {"config": objs})
        for cfg in params["config"]:
            facts["lag_interfaces"].append(utils.remove_empties(cfg))

        ansible_facts["ansible_network_resources"].update(facts)
        return ansible_facts

    def render_config(self, spec, conf, data):
        """
        Render config as dictionary structure and delete keys
        from spec for null values

        :param spec: The facts tree, generated from the argspec
        :param conf: The configuration
        :rtype: dictionary
        :returns: The generated config
        """
        config = deepcopy(spec)
        match = re.search(r"(Bundle-Ether)(\d+)", conf, re.M)
        if match:
            config["name"] = match.group(1) + match.group(2)
            config["load_balancing_hash"] = utils.parse_conf_arg(
                conf,
                "bundle load-balancing hash",
            )
            config["mode"] = utils.parse_conf_arg(conf, "lacp mode")
            config["links"]["max_active"] = utils.parse_conf_arg(
                conf,
                "bundle maximum-active links",
            )
            config["links"]["min_active"] = utils.parse_conf_arg(
                conf,
                "bundle minimum-active links",
            )
            config["members"] = self.parse_members(match.group(2), data)

        return utils.remove_empties(config)

    def parse_members(self, bundle_id, interfaces):
        """
        Renders a list of member interfaces for every bundle
        present in running-config.

        :param bundle_id: The Bundle-Ether ID fetched from running-config
        :param interfaces: Data of all interfaces present in running-config
        :rtype: list
        :returns: A list of member interfaces
        """

        def _parse_interface(name):
            if name.startswith("preconfigure"):
                return name.split()[1]
            else:
                return name.split()[0]

        members = []
        for interface in interfaces:
            if not interface.startswith("Bu"):
                match = re.search(
                    r"bundle id (\d+) mode (\S+)",
                    interface,
                    re.M,
                )
                if match:
                    if bundle_id == match.group(1):
                        members.append(
                            {
                                "member": _parse_interface(interface),
                                "mode": match.group(2),
                            },
                        )

        return members



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: proxy_connection_settings
short_description: Resource module for Proxy Connection Settings
description:
- Manage operation update of the resource Proxy Connection Settings.
- The following functionalities are impacted by the proxy settings.
version_added: '2.1.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  bypassHosts:
    description: Bypass hosts for the proxy connection.
    type: str
  fqdn:
    description: Proxy IP address or DNS-resolvable host name.
    type: str
  password:
    description: Password for the proxy connection.
    type: str
  passwordRequired:
    description: Indicates whether password configuration is required for Proxy.
    type: bool
  port:
    description: Port for proxy connection. Should be between 1 and 65535.
    type: int
  userName:
    description: User name for the proxy connection.
    type: str
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
seealso:
- name: Cisco ISE documentation for proxy
  description: Complete reference of the proxy API.
  link: https://developer.cisco.com/docs/identity-services-engine/v1/#!system-settings-openapi
notes:
  - SDK Method used are
    proxy.Proxy.update_proxy_connection,

  - Paths used are
    put /api/v1/system-settings/proxy,

"""

EXAMPLES = r"""
- name: Update all
  cisco.ise.proxy_connection_settings:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    bypassHosts: string
    fqdn: string
    password: string
    passwordRequired: true
    port: 0
    userName: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "bypassHosts": "string",
      "fqdn": "string",
      "password": "string",
      "passwordRequired": true,
      "port": 0,
      "userName": "string"
    }

ise_update_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "bypassHosts": "string",
        "fqdn": "string",
        "password": "string",
        "passwordRequired": true,
        "port": 0,
        "userName": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: network_access_policy_set
short_description: Resource module for Network Access Policy Set
description:
- Manage operations create, update and delete of the resource Network Access Policy Set.
- Network Access - Create a new policy set.
- Network Access - Delete a policy set.
- Network Access - Update a policy set.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  condition:
    description: Network Access Policy Set's condition.
    suboptions:
      attributeId:
        description: Dictionary attribute id (Optional), used for additional verification.
        type: str
      attributeName:
        description: Dictionary attribute name.
        type: str
      attributeValue:
        description: <ul><li>Attribute value for condition</li> <li>Value type is specified
          in dictionary object</li> <li>if multiple values allowed is specified in dictionary
          object</li></ul>.
        type: str
      children:
        description: In case type is andBlock or orBlock addtional conditions will be
          aggregated under this logical (OR/AND) condition.
        elements: dict
        suboptions:
          conditionType:
            description: <ul><li>Inidicates whether the record is the condition itself(data)
              or a logical(or,and) aggregation</li> <li>Data type enum(reference,single)
              indicates than "conditonId" OR "ConditionAttrs" fields should contain
              condition data but not both</li> <li>Logical aggreation(and,or) enum indicates
              that additional conditions are present under the children field</li></ul>.
            type: str
          isNegate:
            description: Indicates whereas this condition is in negate mode.
            type: bool
          link:
            description: Network Access Policy Set's link.
            suboptions:
              href:
                description: Network Access Policy Set's href.
                type: str
              rel:
                description: Network Access Policy Set's rel.
                type: str
              type:
                description: Network Access Policy Set's type.
                type: str
            type: dict
        type: list
      conditionType:
        description: <ul><li>Inidicates whether the record is the condition itself(data)
          or a logical(or,and) aggregation</li> <li>Data type enum(reference,single)
          indicates than "conditonId" OR "ConditionAttrs" fields should contain condition
          data but not both</li> <li>Logical aggreation(and,or) enum indicates that
          additional conditions are present under the children field</li></ul>.
        type: str
      datesRange:
        description: <p>Defines for which date/s TimeAndDate condition will be matched
          or NOT matched if used in exceptionDates prooperty<br> Options are - Date
          range, for specific date, the same date should be used for start/end date
          <br> Default - no specific dates<br> In order to reset the dates to have no
          specific dates Date format - yyyy-mm-dd (MM = month, dd = day, yyyy = year)</p>.
        suboptions:
          endDate:
            description: Network Access Policy Set's endDate.
            type: str
          startDate:
            description: Network Access Policy Set's startDate.
            type: str
        type: dict
      datesRangeException:
        description: <p>Defines for which date/s TimeAndDate condition will be matched
          or NOT matched if used in exceptionDates prooperty<br> Options are - Date
          range, for specific date, the same date should be used for start/end date
          <br> Default - no specific dates<br> In order to reset the dates to have no
          specific dates Date format - yyyy-mm-dd (MM = month, dd = day, yyyy = year)</p>.
        suboptions:
          endDate:
            description: Network Access Policy Set's endDate.
            type: str
          startDate:
            description: Network Access Policy Set's startDate.
            type: str
        type: dict
      description:
        description: Condition description.
        type: str
      dictionaryName:
        description: Dictionary name.
        type: str
      dictionaryValue:
        description: Dictionary value.
        type: str
      hoursRange:
        description: <p>Defines for which hours a TimeAndDate condition will be matched
          or not matched if used in exceptionHours property<br> Time foramt - hh mm
          ( h = hour , mm = minutes ) <br> Default - All Day </p>.
        suboptions:
          endTime:
            description: Network Access Policy Set's endTime.
            type: str
          startTime:
            description: Network Access Policy Set's startTime.
            type: str
        type: dict
      hoursRangeException:
        description: <p>Defines for which hours a TimeAndDate condition will be matched
          or not matched if used in exceptionHours property<br> Time foramt - hh mm
          ( h = hour , mm = minutes ) <br> Default - All Day </p>.
        suboptions:
          endTime:
            description: Network Access Policy Set's endTime.
            type: str
          startTime:
            description: Network Access Policy Set's startTime.
            type: str
        type: dict
      id:
        description: Network Access Policy Set's id.
        type: str
      isNegate:
        description: Indicates whereas this condition is in negate mode.
        type: bool
      link:
        description: Network Access Policy Set's link.
        suboptions:
          href:
            description: Network Access Policy Set's href.
            type: str
          rel:
            description: Network Access Policy Set's rel.
            type: str
          type:
            description: Network Access Policy Set's type.
            type: str
        type: dict
      name:
        description: Condition name.
        type: str
      operator:
        description: Equality operator.
        type: str
      weekDays:
        description: <p>Defines for which days this condition will be matched<br> Days
          format - Arrays of WeekDay enums <br> Default - List of All week days</p>.
        elements: str
        type: list
      weekDaysException:
        description: <p>Defines for which days this condition will NOT be matched<br>
          Days format - Arrays of WeekDay enums <br> Default - Not enabled</p>.
        elements: str
        type: list
    type: dict
  default:
    description: Flag which indicates if this policy set is the default one.
    type: bool
  description:
    description: The description for the policy set.
    type: str
  hitCounts:
    description: The amount of times the policy was matched.
    type: int
  id:
    description: Identifier for the policy set.
    type: str
  isProxy:
    description: Flag which indicates if the policy set service is of type 'Proxy Sequence'
      or 'Allowed Protocols'.
    type: bool
  link:
    description: Network Access Policy Set's link.
    suboptions:
      href:
        description: Network Access Policy Set's href.
        type: str
      rel:
        description: Network Access Policy Set's rel.
        type: str
      type:
        description: Network Access Policy Set's type.
        type: str
    type: dict
  name:
    description: Given name for the policy set, Valid characters are alphanumerics,
      underscore, hyphen, space, period, parentheses.
    type: str
  rank:
    description: The rank(priority) in relation to other policy set. Lower rank is higher
      priority.
    type: int
  serviceName:
    description: Policy set service identifier - Allowed Protocols,Server Sequence..
    type: str
  state_:
    description: The state that the policy set is in. A disabled policy set cannot be
      matched.
    type: str
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
seealso:
- name: Cisco ISE documentation for Network Access - Policy Set
  description: Complete reference of the Network Access - Policy Set API.
  link: https://developer.cisco.com/docs/identity-services-engine/v1/#!policy-openapi
notes:
  - SDK Method used are
    network_access_policy_set.NetworkAccessPolicySet.create_network_access_policy_set,
    network_access_policy_set.NetworkAccessPolicySet.delete_network_access_policy_set_by_id,
    network_access_policy_set.NetworkAccessPolicySet.update_network_access_policy_set_by_id,

  - Paths used are
    post /network-access/policy-set,
    delete /network-access/policy-set/{id},
    put /network-access/policy-set/{id},

"""

EXAMPLES = r"""
- name: Create
  cisco.ise.network_access_policy_set:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    condition:
      attributeId: string
      attributeName: string
      attributeValue: string
      children:
      - conditionType: string
        isNegate: true
        link:
          href: string
          rel: string
          type: string
      conditionType: string
      datesRange:
        endDate: string
        startDate: string
      datesRangeException:
        endDate: string
        startDate: string
      description: string
      dictionaryName: string
      dictionaryValue: string
      hoursRange:
        endTime: string
        startTime: string
      hoursRangeException:
        endTime: string
        startTime: string
      id: string
      isNegate: true
      link:
        href: string
        rel: string
        type: string
      name: string
      operator: string
      weekDays:
      - string
      weekDaysException:
      - string
    default: true
    description: string
    hitCounts: 0
    id: string
    isProxy: true
    link:
      href: string
      rel: string
      type: string
    name: string
    rank: 0
    serviceName: string
    state_: string

- name: Update by id
  cisco.ise.network_access_policy_set:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    condition:
      attributeId: string
      attributeName: string
      attributeValue: string
      children:
      - conditionType: string
        isNegate: true
        link:
          href: string
          rel: string
          type: string
      conditionType: string
      datesRange:
        endDate: string
        startDate: string
      datesRangeException:
        endDate: string
        startDate: string
      description: string
      dictionaryName: string
      dictionaryValue: string
      hoursRange:
        endTime: string
        startTime: string
      hoursRangeException:
        endTime: string
        startTime: string
      id: string
      isNegate: true
      link:
        href: string
        rel: string
        type: string
      name: string
      operator: string
      weekDays:
      - string
      weekDaysException:
      - string
    default: true
    description: string
    hitCounts: 0
    id: string
    isProxy: true
    link:
      href: string
      rel: string
      type: string
    name: string
    rank: 0
    serviceName: string
    state_: string

- name: Delete by id
  cisco.ise.network_access_policy_set:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: absent
    id: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "condition": {
        "conditionType": "string",
        "isNegate": true,
        "link": {
          "href": "string",
          "rel": "string",
          "type": "string"
        },
        "description": "string",
        "id": "string",
        "name": "string",
        "attributeName": "string",
        "attributeId": "string",
        "attributeValue": "string",
        "dictionaryName": "string",
        "dictionaryValue": "string",
        "operator": "string",
        "children": [
          {
            "conditionType": "string",
            "isNegate": true,
            "link": {
              "href": "string",
              "rel": "string",
              "type": "string"
            }
          }
        ],
        "datesRange": {
          "endDate": "string",
          "startDate": "string"
        },
        "datesRangeException": {
          "endDate": "string",
          "startDate": "string"
        },
        "hoursRange": {
          "endTime": "string",
          "startTime": "string"
        },
        "hoursRangeException": {
          "endTime": "string",
          "startTime": "string"
        },
        "weekDays": [
          "string"
        ],
        "weekDaysException": [
          "string"
        ]
      },
      "default": true,
      "description": "string",
      "hitCounts": 0,
      "id": "string",
      "isProxy": true,
      "link": {
        "href": "string",
        "rel": "string",
        "type": "string"
      },
      "name": "string",
      "rank": 0,
      "serviceName": "string",
      "state": "string"
    }

ise_update_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  version_added: '1.1.0'
  type: dict
  sample: >
    {
      "response": {
        "condition": {
          "conditionType": "string",
          "isNegate": true,
          "link": {
            "href": "string",
            "rel": "string",
            "type": "string"
          },
          "description": "string",
          "id": "string",
          "name": "string",
          "attributeName": "string",
          "attributeId": "string",
          "attributeValue": "string",
          "dictionaryName": "string",
          "dictionaryValue": "string",
          "operator": "string",
          "children": [
            {
              "conditionType": "string",
              "isNegate": true,
              "link": {
                "href": "string",
                "rel": "string",
                "type": "string"
              }
            }
          ],
          "datesRange": {
            "endDate": "string",
            "startDate": "string"
          },
          "datesRangeException": {
            "endDate": "string",
            "startDate": "string"
          },
          "hoursRange": {
            "endTime": "string",
            "startTime": "string"
          },
          "hoursRangeException": {
            "endTime": "string",
            "startTime": "string"
          },
          "weekDays": [
            "string"
          ],
          "weekDaysException": [
            "string"
          ]
        },
        "default": true,
        "description": "string",
        "hitCounts": 0,
        "id": "string",
        "isProxy": true,
        "link": {
          "href": "string",
          "rel": "string",
          "type": "string"
        },
        "name": "string",
        "rank": 0,
        "serviceName": "string",
        "state": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2017 Zim Kalinowski, <zikalino@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_containerinstance
version_added: "0.1.2"
short_description: Manage an Azure Container Instance
description:
    - Create, update and delete an Azure Container Instance.

options:
    resource_group:
        description:
            - Name of resource group.
        type: str
        required: true
    name:
        description:
            - The name of the container group.
        required: true
        type: str
    os_type:
        description:
            - The OS type of containers.
        type: str
        choices:
            - linux
            - windows
        default: linux
    state:
        description:
            - Assert the state of the container instance. Use C(present) to create or update an container instance and C(absent) to delete it.
        type: str
        default: present
        choices:
            - absent
            - present
    ip_address:
        description:
            - The IP address type of the container group.
            - Default is C(none) and creating an instance without public IP.
        type: str
        choices:
            - public
            - none
            - private
        default: 'none'
    dns_name_label:
        description:
            - The Dns name label for the IP.
        type: str
    ports:
        description:
            - List of ports exposed within the container group.
            - This option is deprecated, using I(ports) under I(containers)".
        type: list
        elements: int
        default: []
    location:
        description:
            - Valid azure location. Defaults to location of the resource group.
        type: str
    registry_login_server:
        description:
            - The container image registry login server.
        type: str
    registry_username:
        description:
            - The username to log in container image registry server.
        type: str
    registry_password:
        description:
            - The password to log in container image registry server.
        type: str
    acr_identity:
        description:
            - The Identity to use for access to the registry server.
        type: str
        version_added: '2.5.0'
    containers:
        description:
            - List of containers.
            - Required when creation.
        type: list
        elements: dict
        suboptions:
            name:
                description:
                    - The name of the container instance.
                type: str
                required: true
            image:
                description:
                    - The container image name.
                type: str
                required: true
            memory:
                description:
                    - The required memory of the containers in GB.
                type: float
                default: 1.5
            cpu:
                description:
                    - The required number of CPU cores of the containers.
                type: float
                default: 1
            ports:
                description:
                    - List of ports exposed within the container group.
                type: list
                elements: int
            environment_variables:
                description:
                    - List of container environment variables.
                    - When updating existing container all existing variables will be replaced by new ones.
                type: list
                elements: dict
                suboptions:
                    name:
                        description:
                            - Environment variable name.
                        type: str
                        required: true
                    value:
                        description:
                            - Environment variable value.
                        type: str
                        required: true
                    is_secure:
                        description:
                            - Is variable secure.
                        type: bool
            volume_mounts:
                description:
                    - The volume mounts for the container instance
                type: list
                elements: dict
                suboptions:
                    name:
                        description:
                            - The name of the volume mount
                        required: true
                        type: str
                    mount_path:
                        description:
                            - The path within the container where the volume should be mounted
                        required: true
                        type: str
                    read_only:
                        description:
                            - The flag indicating whether the volume mount is read-only
                        type: bool
            commands:
                description:
                    - List of commands to execute within the container instance in exec form.
                    - When updating existing container all existing commands will be replaced by new ones.
                type: list
                elements: str
    restart_policy:
        description:
            - Restart policy for all containers within the container group.
        type: str
        choices:
            - always
            - on_failure
            - never
    subnet_ids:
        description:
            - The subnet resource IDs for a container group.
            - Multiple subnets are not yet supported. Only 1 subnet can be used.
        type: list
        elements: str
    volumes:
        description:
            - List of Volumes that can be mounted by containers in this container group.
        type: list
        elements: dict
        suboptions:
            name:
                description:
                    - The name of the Volume
                required: true
                type: str
            azure_file:
                description:
                    - The Azure File volume
                type: dict
                suboptions:
                    share_name:
                        description:
                            - The name of the Azure File share to be mounted as a volume
                        required: true
                        type: str
                    read_only:
                        description:
                            - The flag indicating whether the Azure File shared mounted as a volume is read-only
                        type: bool
                    storage_account_name:
                        description:
                            - The name of the storage account that contains the Azure File share
                        required: true
                        type: str
                    storage_account_key:
                        description:
                            - The storage account access key used to access the Azure File share
                        required: true
                        type: str
            empty_dir:
                description:
                    - The empty directory volume
                type: dict
            secret:
                description:
                    - The secret volume
                type: dict
            git_repo:
                description:
                    - The git repo volume
                type: dict
                suboptions:
                    directory:
                        description:
                            - Target directory name
                        type: str
                    repository:
                        description:
                            - Repository URL
                        required: true
                        type: str
                    revision:
                        description:
                            - Commit hash for the specified revision
                        type: str
    identity:
        description:
            - Identity for the Server.
        type: dict
        version_added: '2.5.0'
        suboptions:
            type:
                description:
                    - Type of the managed identity
                required: false
                choices:
                    - SystemAssigned
                    - SystemAssigned, UserAssigned
                    - UserAssigned
                default: None
                type: str
            user_assigned_identities:
                description:
                    - User Assigned Managed Identities and its options
                required: false
                type: dict
                default: {}
                suboptions:
                    id:
                        description:
                            - List of the user assigned identities IDs associated to the VM
                        required: false
                        type: list
                        elements: str
                        default: []
    force_update:
        description:
            - Force update of existing container instance. Any update will result in deletion and recreation of existing containers.
        type: bool
        default: 'no'

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Zim Kalinowski (@zikalino)

'''

EXAMPLES = '''
- name: Create sample container group
  azure_rm_containerinstance:
    resource_group: myResourceGroup
    name: myContainerInstanceGroup
    os_type: linux
    ip_address: public
    containers:
      - name: myContainer1
        image: httpd
        memory: 1.5
        ports:
          - 80
          - 81

- name: Create sample container group with azure file share volume
  azure_rm_containerinstance:
    resource_group: myResourceGroup
    name: myContainerInstanceGroupz
    os_type: linux
    ip_address: public
    containers:
      - name: mycontainer1
        image: httpd
        memory: 1
        volume_mounts:
          - name: filesharevolume
            mount_path: "/data/files"
        ports:
          - 80
          - 81
    volumes:
      - name: filesharevolume
        azure_file:
          storage_account_name: mystorageaccount
          share_name: acishare
          storage_account_key: xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

- name: Create sample container group with git repo volume
  azure_rm_containerinstance:
    resource_group: myResourceGroup
    name: myContainerInstanceGroup
    os_type: linux
    ip_address: public
    containers:
      - name: mycontainer1
        image: httpd
        memory: 1
        volume_mounts:
          - name: myvolume1
            mount_path: "/mnt/test"
        ports:
          - 80
          - 81
    volumes:
      - name: myvolume1
        git_repo:
          repository: "https://github.com/Azure-Samples/aci-helloworld.git"

- name: Create sample container instance with subnet
  azure_rm_containerinstance:
    resource_group: myResourceGroup
    name: myContainerInstanceGroup
    os_type: linux
    ip_address: private
    location: eastus
    subnet_ids:
      - "{{ subnet_id }}"
    ports:
      - 80
    containers:
      - name: mycontainer1
        image: httpd
        memory: 1.5
        ports:
          - 80
          - 81
'''
RETURN = '''
id:
    description:
        - Resource ID.
    returned: always
    type: str
    sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.ContainerInstance/containerGroups/aci1b6dd89
provisioning_state:
    description:
        - Provisioning state of the container.
    returned: always
    type: str
    sample: Creating
ip_address:
    description:
        - Public IP Address of created container group.
    returned: if address is public
    type: str
    sample: 175.12.233.11
containers:
    description:
        - The containers within the container group.
    returned: always
    type: list
    elements: dict
    sample: [
                {
                    "commands": null,
                    "cpu": 1.0,
                    "environment_variables": null,
                    "image": "httpd",
                    "memory": 1.0,
                    "name": "mycontainer1",
                    "ports": [
                        80,
                        81
                    ],
                    "volume_mounts": [
                        {
                            "mount_path": "/data/files",
                            "name": "filesharevolume",
                            "read_only": false
                        }
                    ]
                }
    ]
volumes:
    description:
        - The list of volumes that mounted by containers in container group
    returned: if volumes specified
    type: list
    elements: dict
    contains:
        name:
            description:
                - The name of the Volume
            returned: always
            type: str
            sample: filesharevolume
        azure_file:
            description:
                - Azure file share volume details
            returned: If Azure file share type of volume requested
            type: dict
            sample: {
                        "read_only": null,
                        "share_name": "acishare",
                        "storage_account_key": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                        "storage_account_name": "mystorageaccount"
            }
        empty_dir:
            description:
                - Empty directory volume details
            returned: If Empty directory type of volume requested
            type: dict
            sample: {}
        secret:
            description:
                - Secret volume details
            returned: If Secret type of volume requested
            type: dict
            sample: {}
        git_repo:
            description:
                - Git Repo volume details
            returned: If Git repo type of volume requested
            type: dict
            sample: {
                        "directory": null,
                        "repository": "https://github.com/Azure-Samples/aci-helloworld.git",
                        "revision": null
            }
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt
from ansible.module_utils.common.dict_transformations import _snake_to_camel

try:
    from azure.core.exceptions import ResourceNotFoundError
    from azure.core.polling import LROPoller
    from azure.mgmt.containerinstance.models import (
        ContainerGroupIdentity,
    )
except ImportError:
    # This is handled in azure_rm_common
    pass


def create_container_dict_from_obj(container):
    '''
    Create a dict from an instance of a Container.

    :param rule: Container
    :return: dict
    '''
    results = dict(
        name=container.name,
        image=container.image,
        memory=container.resources.requests.memory_in_gb,
        cpu=container.resources.requests.cpu
        # command (list of str)
        # ports (list of ContainerPort)
        # environment_variables (list of EnvironmentVariable)
        # resources (ResourceRequirements)
        # volume mounts (list of VolumeMount)
    )

    if container.instance_view is not None:
        # instance_view (ContainerPropertiesInstanceView)
        results["instance_restart_count"] = container.instance_view.restart_count
        if container.instance_view.current_state:
            results["instance_current_state"] = container.instance_view.current_state.state
            results["instance_current_start_time"] = container.instance_view.current_state.start_time
            results["instance_current_exit_code"] = container.instance_view.current_state.exit_code
            results["instance_current_finish_time"] = container.instance_view.current_state.finish_time
            results["instance_current_detail_status"] = container.instance_view.current_state.detail_status
        if container.instance_view.previous_state:
            results["instance_previous_state"] = container.instance_view.previous_state.state
            results["instance_previous_start_time"] = container.instance_view.previous_state.start_time
            results["instance_previous_exit_code"] = container.instance_view.previous_state.exit_code
            results["instance_previous_finish_time"] = container.instance_view.previous_state.finish_time
            results["instance_previous_detail_status"] = container.instance_view.previous_state.detail_status
        # events (list of ContainerEvent)
    return results


env_var_spec = dict(
    name=dict(type='str', required=True),
    value=dict(type='str', required=True),
    is_secure=dict(type='bool')
)


volume_mount_var_spec = dict(
    name=dict(type='str', required=True),
    mount_path=dict(type='str', required=True),
    read_only=dict(type='bool')
)


container_spec = dict(
    name=dict(type='str', required=True),
    image=dict(type='str', required=True),
    memory=dict(type='float', default=1.5),
    cpu=dict(type='float', default=1),
    ports=dict(type='list', elements='int'),
    commands=dict(type='list', elements='str'),
    environment_variables=dict(type='list', elements='dict', options=env_var_spec),
    volume_mounts=dict(type='list', elements='dict', options=volume_mount_var_spec)
)


git_repo_volume_spec = dict(
    directory=dict(type='str'),
    repository=dict(type='str', required=True),
    revision=dict(type='str')
)


azure_file_volume_spec = dict(
    share_name=dict(type='str', required=True),
    read_only=dict(type='bool'),
    storage_account_name=dict(type='str', required=True),
    storage_account_key=dict(type='str', required=True, no_log=True)
)


volumes_spec = dict(
    name=dict(type='str', required=True),
    azure_file=dict(type='dict', options=azure_file_volume_spec),
    empty_dir=dict(type='dict'),
    secret=dict(type='dict', no_log=True),
    git_repo=dict(type='dict', options=git_repo_volume_spec)
)


user_assigned_identities_spec = dict(
    id=dict(
        type='list',
        default=[],
        elements='str'
    ),
)

managed_identity_spec = dict(
    type=dict(
        type='str',
        choices=['SystemAssigned',
                 'UserAssigned',
                 'SystemAssigned, UserAssigned'],
        default='None'
    ),
    user_assigned_identities=dict(
        type='dict',
        options=user_assigned_identities_spec,
        default={}
    ),
)


class AzureRMContainerInstance(AzureRMModuleBaseExt):
    """Configuration class for an Azure RM container instance resource"""

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True
            ),
            os_type=dict(
                type='str',
                default='linux',
                choices=['linux', 'windows']
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            ),
            location=dict(
                type='str',
            ),
            ip_address=dict(
                type='str',
                default='none',
                choices=['public', 'none', 'private']
            ),
            dns_name_label=dict(
                type='str',
            ),
            ports=dict(
                type='list',
                elements='int',
                default=[]
            ),
            registry_login_server=dict(
                type='str',
                default=None
            ),
            registry_username=dict(
                type='str',
                default=None
            ),
            registry_password=dict(
                type='str',
                default=None,
                no_log=True
            ),
            acr_identity=dict(
                type='str',
                default=None,
            ),
            containers=dict(
                type='list',
                elements='dict',
                options=container_spec
            ),
            restart_policy=dict(
                type='str',
                choices=['always', 'on_failure', 'never']
            ),
            force_update=dict(
                type='bool',
                default=False
            ),
            volumes=dict(
                type='list',
                elements='dict',
                options=volumes_spec
            ),
            subnet_ids=dict(
                type='list',
                elements='str',
            ),
            identity=dict(
                type='dict',
                options=managed_identity_spec
            ),
        )

        self.resource_group = None
        self.name = None
        self.location = None
        self.state = None
        self.ip_address = None
        self.dns_name_label = None
        self.containers = None
        self.restart_policy = None
        self.subnet_ids = None
        self.identity = None

        self.tags = None

        self.results = dict(changed=False, state=dict())
        self.cgmodels = None
        self._managed_identity = None

        required_if = [
            ('state', 'present', ['containers']), ('ip_address', 'private', ['subnet_ids'])
        ]

        super(AzureRMContainerInstance, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                       supports_check_mode=True,
                                                       supports_tags=True,
                                                       required_if=required_if)

    @property
    def managed_identity(self):
        if not self._managed_identity:
            self._managed_identity = {"identity": ContainerGroupIdentity,
                                      "user_assigned": dict
                                      }
        return self._managed_identity

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        resource_group = None
        response = None
        results = dict()

        # since this client hasn't been upgraded to expose models directly off the OperationClass, fish them out
        self.cgmodels = self.containerinstance_client.container_groups.models

        resource_group = self.get_resource_group(self.resource_group)

        if not self.location:
            self.location = resource_group.location

        response = self.get_containerinstance()

        # Format identities
        if self.identity:
            update_identity, self.identity = self.update_managed_identity(new_identity=self.identity)

        if not response:
            self.log("Container Group doesn't exist")

            if self.state == 'absent':
                self.log("Nothing to delete")
            else:
                self.force_update = True
        else:
            self.log("Container instance already exists")

            if self.state == 'absent':
                if not self.check_mode:
                    self.delete_containerinstance()
                self.results['changed'] = True
                self.log("Container instance deleted")
            elif self.state == 'present':
                self.log("Need to check if container group has to be deleted or may be updated")
                update_tags, newtags = self.update_tags(response.get('tags', dict()))

                if self.force_update:
                    self.log('Deleting container instance before update')
                    if not self.check_mode:
                        self.delete_containerinstance()
                elif update_tags:
                    if not self.check_mode:
                        self.tags = newtags
                        self.results['changed'] = True
                        response = self.update_containerinstance()

        if self.state == 'present':

            self.log("Need to Create / Update the container instance")

            if self.force_update:
                self.results['changed'] = True
                if self.check_mode:
                    return self.results
                response = self.create_update_containerinstance()

            self.results['id'] = response['id']
            self.results['provisioning_state'] = response['provisioning_state']
            self.results['ip_address'] = response['ip_address']['ip'] if 'ip_address' in response else ''

            self.log("Creation / Update done")

        return self.results

    def update_containerinstance(self):
        '''
        Updates a container service with the specified configuration of orchestrator, masters, and agents.

        :return: deserialized container instance state dictionary
        '''
        try:
            response = self.containerinstance_client.container_groups.update(resource_group_name=self.resource_group,
                                                                             container_group_name=self.name,
                                                                             resource=dict(tags=self.tags))
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
        except Exception as exc:
            self.fail("Error when Updating ACI {0}: {1}".format(self.name, exc.message or str(exc)))

        return response.as_dict()

    def create_update_containerinstance(self):
        '''
        Creates or updates a container service with the specified configuration of orchestrator, masters, and agents.

        :return: deserialized container instance state dictionary
        '''
        self.log("Creating / Updating the container instance {0}".format(self.name))

        registry_credentials = None

        if self.registry_login_server is not None:
            registry_credentials = [self.cgmodels.ImageRegistryCredential(server=self.registry_login_server,
                                                                          username=self.registry_username,
                                                                          password=self.registry_password,
                                                                          identity=self.acr_identity)]

        ip_address = None

        containers = []
        all_ports = set([])
        for container_def in self.containers:
            name = container_def.get("name")
            image = container_def.get("image")
            memory = container_def.get("memory")
            cpu = container_def.get("cpu")
            commands = container_def.get("commands")
            ports = []
            variables = []
            volume_mounts = []

            port_list = container_def.get("ports")
            if port_list:
                for port in port_list:
                    all_ports.add(port)
                    ports.append(self.cgmodels.ContainerPort(port=port))

            variable_list = container_def.get("environment_variables")
            if variable_list:
                for variable in variable_list:
                    variables.append(self.cgmodels.EnvironmentVariable(name=variable.get('name'),
                                                                       value=variable.get('value') if not variable.get('is_secure') else None,
                                                                       secure_value=variable.get('value') if variable.get('is_secure') else None))

            volume_mounts_list = container_def.get("volume_mounts")
            if volume_mounts_list:
                for volume_mount in volume_mounts_list:
                    volume_mounts.append(self.cgmodels.VolumeMount(name=volume_mount.get('name'),
                                                                   mount_path=volume_mount.get('mount_path'),
                                                                   read_only=volume_mount.get('read_only')))

            containers.append(self.cgmodels.Container(name=name,
                                                      image=image,
                                                      resources=self.cgmodels.ResourceRequirements(
                                                          requests=self.cgmodels.ResourceRequests(memory_in_gb=memory, cpu=cpu)
                                                      ),
                                                      ports=ports,
                                                      command=commands,
                                                      environment_variables=variables,
                                                      volume_mounts=volume_mounts))

        if self.ip_address is not None:
            # get list of ports
            if len(all_ports) > 0:
                ports = []
                for port in all_ports:
                    ports.append(self.cgmodels.Port(port=port, protocol="TCP"))
                ip_address = self.cgmodels.IpAddress(ports=ports, dns_name_label=self.dns_name_label, type=self.ip_address)

        subnet_ids = None
        if self.subnet_ids is not None:
            subnet_ids = [self.cgmodels.ContainerGroupSubnetId(id=item) for item in self.subnet_ids]

        parameters = self.cgmodels.ContainerGroup(location=self.location,
                                                  identity=self.identity,
                                                  containers=containers,
                                                  image_registry_credentials=registry_credentials,
                                                  restart_policy=_snake_to_camel(self.restart_policy, True) if self.restart_policy else None,
                                                  ip_address=ip_address,
                                                  os_type=self.os_type,
                                                  subnet_ids=subnet_ids,
                                                  volumes=self.volumes,
                                                  tags=self.tags)

        try:
            response = self.containerinstance_client.container_groups.begin_create_or_update(resource_group_name=self.resource_group,
                                                                                             container_group_name=self.name,
                                                                                             container_group=parameters)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
        except Exception as exc:
            self.fail("Error when creating ACI {0}: {1}".format(self.name, exc.message or str(exc)))

        return response.as_dict()

    def delete_containerinstance(self):
        '''
        Deletes the specified container group instance in the specified subscription and resource group.

        :return: True
        '''
        self.log("Deleting the container instance {0}".format(self.name))
        try:
            response = self.containerinstance_client.container_groups.begin_delete(resource_group_name=self.resource_group, container_group_name=self.name)
            return True
        except Exception as exc:
            self.fail('Error when deleting ACI {0}: {1}'.format(self.name, exc.message or str(exc)))
            return False

    def get_containerinstance(self):
        '''
        Gets the properties of the specified container service.

        :return: deserialized container instance state dictionary
        '''
        self.log("Checking if the container instance {0} is present".format(self.name))
        found = False
        try:
            response = self.containerinstance_client.container_groups.get(resource_group_name=self.resource_group, container_group_name=self.name)
            found = True
            self.log("Response : {0}".format(response))
            self.log("Container instance : {0} found".format(response.name))
        except ResourceNotFoundError as e:
            self.log('Did not find the container instance.')
        if found is True:
            return response.as_dict()

        return False


def main():
    """Main execution"""
    AzureRMContainerInstance()


if __name__ == '__main__':
    main()
